/* Declare and implement CArray.

This class implements a dynamic alocated array of templates.

FEATURES:
- you can realocate the array mantaning all data (if no loose of information)
  using CArray::mRealocate. Data is truncated if array is smaller than the
  original array.
- you can use (void*) and others cast operators to convert array data and
  access array data directly.

TIPS:
- the template can be any data type. To declare an array of char use:
  CArray <char> oArray;
- the index of first element is 0
- the index of last element is CArray::m_dwSize - 1
*/



// verify if has been declared, if no do it
#ifndef __3DFC_ARRAY_H__



// signals file inclusion
#define __3DFC_ARRAY_H__



template < class TEMPLATE > class CArray {

public:

   TEMPLATE* m_paArray;      // point data buffer
   DWORD m_dwSize;           // number of elements in buffer



/* constructor - init empty array
**********************************************************/
CArray ()
{
   m_paArray = NULL;
   m_dwSize = 0;
}



/* constructor - alocate a specific size array
**********************************************************/
CArray ( DWORD dwSize )
{
   m_paArray = NULL;
   m_dwSize = 0;

   mCreate ( dwSize );
}



/* constructor - init array with a copy of other array
**********************************************************/
CArray ( CArray& oArray )
{
   m_paArray = NULL;
   m_dwSize = 0;

   mCreate ( oArray.m_dwSize );

   memcpy ( (void*) m_paArray, (void*) oArray.m_paArray, m_dwSize * sizeof ( TEMPLATE ) );
}



/* destructor - desalocate array, if alocated
**********************************************************/
~CArray ()
{
   mDestroy ();
}



/* Create array doing dinamic alocation. If not enough memory
abort program execution. (like constructor)
**********************************************************/
void mCreate ( DWORD dwSize )
{
   mDestroy ();

   // try alocation
   if ( ( m_paArray = new TEMPLATE [ dwSize ] ) == NULL )
      ABORT ( MSG_ERROR_MEMORY_LOW, __FILE__, __LINE__ )

   m_dwSize = dwSize;
}



/* Desalocate array, if alocated
**********************************************************/
void mDestroy ()
{
   // verify if it's alocated
   if ( m_paArray != NULL ) {
      delete m_paArray;
      m_dwSize = 0;
      m_paArray = NULL;
   }
}



/* Realocate array, copy the original elements to the new array.
If new array is smaller than the original buffer they are truncated.
dwSize - the size for the new array.
**********************************************************/
void mRealocate ( DWORD dwSize )
{
   // Create new memory array
   // try alocation
   TEMPLATE
      *paArray;
   if ( ( paArray = new TEMPLATE [ dwSize ] ) == NULL )
      ABORT ( MSG_ERROR_MEMORY_LOW, __FILE__, __LINE__ )

   // copy elements to new array
   // if new array is short than the actual: truncate
   if ( dwSize < m_dwSize )
      memcpy ( (void*) paArray, (void*) m_paArray, dwSize * sizeof ( TEMPLATE ) );
   else
      // if new array have igual size or is greater: copy all
      memcpy ( (void*) paArray, (void*) m_paArray, m_dwSize * sizeof ( TEMPLATE ) );

   // desalocate current array
   if ( m_paArray != NULL )
      delete m_paArray;

   // change current array to new
   m_paArray = paArray;
   m_dwSize = dwSize;
}



/* Init memory array if copy of other object
(like constructor and operator =)
**********************************************************/
void mSetObj ( CArray& oArray )
{
   mCreate ( oArray.m_dwSize );

   memcpy ( (void*) m_paArray, (void*) oArray.m_paArray, m_dwSize * sizeof ( TEMPLATE ) );
}



/* like constructor and mSetObj
**********************************************************/
void operator = ( CArray& oArray )
{
   mSetObj ( oArray );
}



/* Return an element
**********************************************************/
TEMPLATE operator [] ( DWORD dwPos )
{
   return * ( m_paArray + dwPos );
}



/* Cast object to byte array, return memory array
**********************************************************/
operator BYTE* ()
{
   return (BYTE*) m_paArray;
}



/* Cast object to TEMPLATE array, return memory array
**********************************************************/
operator TEMPLATE* ()
{
   return m_paArray;
}



/* Cast object to void array, return memory array
**********************************************************/
operator void* ()
{
   return (void*) m_paArray;
}



};



// end declarations
#endif
