/* Some drawing macros for scanline conversion and stuff.
Used by src/draw.cc
The asm instructions must end with ";" or "\n" to
expand macro correctly.

Note:
A simplification:
   256 * ( nTexY >> 16 ) ==
   ( nTexY >> 16 ) << 16  ==
   ( nTexY >> 8 ) & 0xFF00          - is more fast ?!
*/



/* Flat horizontal line draw
(draw 4 bytes in same time, use strings instructions).
DESTIN - is a destin pointer
LENGHT - line lenght
COLOR - line color

Remarks:
   movl  %0,      %%edi    destin
   movl  %1,      %%ecx    number of interactions (line length)
   movb  %2,      %%eax    color (repeat color 4 times - 32 bits)
                           If length is not multiple of 4 draw line by
                           4 in 4 bytes and the rest. Start drawing rest.
   movb  %%cl,    %%bl     load bh with last 8 bits to test
   andb  $3,      %%bl     select only 2 last bits (the rest)
   cmpb  $3,      %%bl     verify if rest is 3, if yes draw 3 pixels
   je    Draw3             goto draw continuous
   cmpb  $2,      %%bl     verify if rest is 2, if yes draw 2 pixels
   je    Draw2             goto draw continuous
   cmpb  $1,      %%bl     verify if rest is 1, if yes draw 1 pixels
   je    Draw1             goto draw continuous
   jmp   DrawAll           if no goto executed the rest is 0
Draw3:                     if rest 3
   stosb
Draw2:                     if rest 2 or 3
   stosb
Draw1:                     if rest 3 or 2 or 1
   stosb

DrawAll:                   draw line (do not test rest)
   shrl  $2,     %%ecx     remove interactions done in rest cases (/4)
   cld                     clear direction flag
   rep                     init repet until ecx equal 0 (end of line)
      stosl                put 4 pixels (the value in eax)

**********************************************************/
#define ASM_HLINE_FLAT(DESTIN, LENGHT, COLOR) \
__asm__ __volatile__ ("                 \
                                        \
      movl  %0,      %%edi;             \
      movl  %1,      %%ecx;             \
      movl  %2,      %%eax;             \
                                        \
      movb  %%cl,    %%bl;               \
      andb  $3,      %%bl;              \
      cmpb  $3,      %%bl;              \
      je    DrawPix3;                   \
      cmpb  $2,      %%bl;              \
      je    DrawPix2;                   \
      cmpb  $1,      %%bl;              \
      je    DrawPix1;                   \
      jmp   DrawPixAll;                  \
   DrawPix3:;                            \
      stosb;                             \
   DrawPix2:;                            \
      stosb;                             \
   DrawPix1:;                             \
      stosb;                              \
                                          \
   DrawPixAll:;                          \
      shrl  $2,      %%ecx;               \
      cld;                                \
      rep;                                \
         stosl;                            \
      "                                    \
      :                                     \
      : "g" ( ( DESTIN ) ), "g" ( ( LENGHT ) ), "g" ( ( COLOR ) ) \
      : "%bl", "cl", "%ecx", "%edi", "eax"            \
);



/* Redeclare ASM_HLINE_FLAT because is impossible expand the macro
more then one time (asm labels must be unique in file!)
**********************************************************/
#define ASM_HLINE_FLAT_2(DESTIN, LENGHT, COLOR) \
__asm__ __volatile__ ("                 \
                                        \
      movl  %0,      %%edi;             \
      movl  %1,      %%ecx;             \
      movl  %2,      %%eax;             \
                                        \
      movb  %%cl,    %%bl;               \
      andb  $3,      %%bl;              \
      cmpb  $3,      %%bl;              \
      je    DrawPix3_2;                   \
      cmpb  $2,      %%bl;              \
      je    DrawPix2_2;                   \
      cmpb  $1,      %%bl;              \
      je    DrawPix1_2;                   \
      jmp   DrawPixAll_2;                  \
   DrawPix3_2:;                            \
      stosb;                             \
   DrawPix2_2:;                            \
      stosb;                             \
   DrawPix1_2:;                             \
      stosb;                              \
                                          \
   DrawPixAll_2:;                          \
      shrl  $2,      %%ecx;               \
      cld;                                \
      rep;                                \
         stosl;                            \
      "                                    \
      :                                     \
      : "g" ( ( DESTIN ) ), "g" ( ( LENGHT ) ), "g" ( ( COLOR ) ) \
      : "%bl", "cl", "%ecx", "%edi", "eax"            \
);



/* Copy one texel from source texture bitmap to frame buffer
DESTIN - point the byte where write data
SOURCE - point the texture bitmap
COORD_TEX_X, COORD_TEX_Y - texture coordenates in bitmap

Remarks:
movl  %2,       %%esi    X
shrl  $16,      %%esi    fix to int

movl  %3,       %%eax    Y
shrl  $8,       %%eax    make the simplification to convert fix to int
                         and multiply (remarks at the begin of the file)
andl  $65280,   %%eax    has found the line in array

addl  %%eax,    %%esi    offset in array
addl  %1,       %%esi    affset address in memory

movb  (%%esi),  %%al     load texel

movb  %%al,    %0"       store texel

**********************************************************/
#define ASM_TRANSFER_TEXEL(DESTIN, SOURCE, COORD_TEX_X, COORD_TEX_Y) \
__asm__ __volatile__ ("    \
   movl  %2,       %%esi;  \
   shrl  $16,      %%esi;  \
                           \
   movl  %3,       %%eax;  \
   shrl  $8,       %%eax;  \
   andl  $65280,   %%eax;  \
                           \
   addl  %%eax,    %%esi;  \
   addl  %1,       %%esi;  \
                           \
   movb  (%%esi),  %%al;   \
   movb  %%al,    %0;"     \
   : "=g" ( *DESTIN )       \
   : "g" ( SOURCE ), "g" ( COORD_TEX_X ), "g" ( COORD_TEX_Y ) \
   : "eax", "esi"                                             \
);



/* Copy one texel lighted.
Get one texel color from source texture bitmap and use it as index to
find the lighted texel in light table, the light level is given, too.
DESTIN - point the byte where write data
SOURCE_TEX - point the texture bitmap
COORD_TEX_X, COORD_TEX_Y - texture coordenates in bitmap (fixed: 16.16)
SOURCE_LTAB - point the source light table
LIGHT_LEVEL - the light level (fixed: 16.16)

Textures are square: X, Y.
Along X axis (X2), in light table, is the complete palette colors
for one light level. Along Y axis (Y2) is all light levels for one
color.

Remarks:
movl  %2,       %%esi;     texel X
shrl  $16,      %%esi;     convert fix int (>>16)

movl  %3,       %%eax;     texel Y
shrl  $8,       %%eax;     convert fix int e multiply
andl  $65280,   %%eax;     use simplified calc, see begin of the file

addl  %%eax,    %%esi;     array offset
addl  %1,       %%esi;     memory offset

movl  %5,       %%eax;     read light table line (Y2)
shrl  $8,       %%eax;     convert fix int e multiply
andl  $65280,   %%eax;     use simplified calc, see begin of the file
addl  %4,       %%eax;     line offset in memory

xorl  %%ebx,    %%ebx;     zero row (color information)
movb  (%%esi),  %%bl;      read row: X2
addl  %%ebx,    %%eax;     offset in memory (for lighted color)

movl  %%eax,    %%esi;     copy source address (prepare for read)
movb  (%%esi),  %%al;      read lighted color
movb  %%al,     %0;        write to destin

**********************************************************/
#define ASM_TRANSFER_LIGHTED_TEXEL(DESTIN, SOURCE_TEX, COORD_TEX_X, COORD_TEX_Y,SOURCE_LTAB,LIGHT_LEVEL) \
__asm__ __volatile__ ("        \
      movl  %2,       %%esi;   \
      shrl  $16,      %%esi;   \
                               \
      movl  %3,       %%eax;   \
      shrl  $8,       %%eax;   \
      andl  $65280,   %%eax;   \
                               \
      addl  %%eax,    %%esi;   \
      addl  %1,       %%esi;   \
                               \
      movl  %5,       %%eax;   \
      shrl  $8,       %%eax;   \
      andl  $65280,   %%eax;   \
      addl  %4,       %%eax;    \
                               \
      xorl  %%ebx,    %%ebx;   \
      movb  (%%esi),  %%bl;    \
      addl  %%ebx,    %%eax;   \
                               \
      movl  %%eax,    %%esi;   \
      movb  (%%esi),  %%al;    \
      movb  %%al,     %0;"     \
                                \
   : "=g" ( *DESTIN )           \
   : "g" ( SOURCE_TEX ), "g" ( COORD_TEX_X ), "g" ( COORD_TEX_Y ), "g" ( SOURCE_LTAB ), "g" ( LIGHT_LEVEL ) \
   : "eax", "esi", "bl"                                                                                    \
);
