/* Implements a camera reference system with some transformations and
camera data.
Example:
             J
            |
            |  / K
            | /
            |/
            *------
             O     I

TIPS:
- this class is derived from CReferenceSystem, see refsys.cc for more
  information.
*/



#include <string.h>

#include <math.h>

#include "defs.h"

#include "node.h"

#include "list.h"

#include "polygon.h"

#include "nodepol.h"

#include "nodexyz.h"

#include "nodexy.h"

#include "math.h"

#include "refsys.h"

#include "camsys.h"



/* constructor - init default reference system
bBuildTrigTables - TRUE - create sin/cos lookup tables
(default FALSE)
**********************************************************/
CCameraSystem::CCameraSystem ( BOOL bBuildTrigTables )
{
   mCreate ( bBuildTrigTables );
}



/* destructor
**********************************************************/
CCameraSystem::~CCameraSystem ()
{
}



/* Init reference system and lockup tables
bBuildTrigTables - TRUE - create sin/cos lookup tables
(default FALSE)
**********************************************************/
void CCameraSystem::mCreate ( BOOL bBuildTrigTables )
{
   CReferenceSystem::mCreate ( bBuildTrigTables );

   m_fDistCP = DIST_CP;
   m_fRadius = DEFAULT_RADIUS;   
   m_fSpeed = DEFAULT_MAXIMUM_SPEED;
}



/* Destroy object
**********************************************************/
void CCameraSystem::mDestroy ()
{
   CReferenceSystem::mDestroy ();
}



/* Init reference system with a copy of other object
(like CCameraSystem::constructor and CCameraSystem::operator =)
**********************************************************/
void CCameraSystem::mSetObj ( CCameraSystem& oCS )
{
   CReferenceSystem::mSetObj ( oCS );

   // distance of projection center and projection plane
   m_fDistCP = oCS.m_fDistCP;
   // radius
   m_fRadius = oCS.m_fRadius;
   // move speed
   m_fSpeed = oCS.m_fSpeed;
}



/* like CCameraSystem::constructor and CCameraSystem::mSetObj
**********************************************************/
void CCameraSystem::operator = ( CCameraSystem& oCS )
{
   mSetObj ( oCS );
}



/* Invert camera to use it as the view camera system or as
an object RS.
**********************************************************/
void CCameraSystem::mInvertCamera ()
{
   // invert K
   m_stK.fX = - m_stK.fX;
   m_stK.fY = - m_stK.fY;
   m_stK.fZ = - m_stK.fZ;
   // update position
   m_stPos.fX = - m_stPos.fX;
   m_stPos.fY = - m_stPos.fY;
   m_stPos.fZ = - m_stPos.fZ;
}
