/* Declare a component (activable) with a general data storage pointer.
The data can be, for example: a CMesh instance. */



#include "../defs.h"
      
#include "../node.h"

#include "../list.h"

#include "../map.h"

#include "../polygon.h"

#include "../nodepol.h"

#include "../nodeint.h"

#include "../nodeflo.h"

#include "../nodexyz.h"

#include "../math.h"

#include "../mesh.h"

#include "../refsys.h"

#include "../camsys.h"

#include "nodecomp.h"

#include "codata.h"



/* contructor - init with default component type
**********************************************************/
CComponentData::CComponentData ()
{
   m_dwMyTypeID = TYPE_DATA;
   m_dwCastID = CAST_NONE;
   m_paData = NULL;
   m_dwNumElements = 0;
}



/* destructor - desalocate data
**********************************************************/
CComponentData::~CComponentData ()
{
   mDestroy ( 0, 0, 0 );
}



/* Alocate data
pData1 - define the data to alocate (the cast ID)
pData2 - the number of units to alocate
pData3 - not used
return value 0 - ok / value 1 - error
**********************************************************/
PTRVAL CComponentData::mCreate ( PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 )
{
   pData3 = pData3;

   mDestroy ( 0, 0, 0 );

   // alocate
   switch ( pData1 ) {

      case CAST_BYTE:
         m_paData = (void*) new BYTE [ pData2 ];
         break;

      case CAST_DWORD:
         m_paData = (void*) new DWORD [ pData2 ];
         break;

      case CAST_FLOAT:
         m_paData = (void*) new float [ pData2 ];
         break;

      case CAST_CMESH:
         m_paData = (void*) new CMesh [ pData2 ];
         break;

      case CAST_CCAMERASYSTEM:
         m_paData = (void*) new CCameraSystem [ pData2 ];
         break;

      default:
         ABORT ( MSG_ERROR_NO_CAST, __FILE__, __LINE__ );
         break;
   }

   // verify if not alocated
   if ( m_paData == NULL )
      return 1;

   // update data
   m_dwCastID = pData1;
   m_dwNumElements = pData2;

   return 0;
}



/* Desalocate data
The given parameters have no use.
**********************************************************/
PTRVAL CComponentData::mDestroy ( PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 )
{
   pData1 = pData1;        // to avoid GCC warnings
   pData2 = pData2;        // to avoid GCC warnings
   pData3 = pData3;        // to avoid GCC warnings
   
   if ( m_paData != NULL ) {
      delete m_paData;
      m_paData = NULL;
      m_dwNumElements = 0;
   }

   return 0;
}
