/* Make camera move, reading keyboard.

TIPS:
- this is one possible object component.
- this class derive from CNodeComponent, see nodecomp.h for more info.
*/



#include <string.h>

#include "../defs.h"

#include "../mem.h"

#include "../array.h"

#include "../string.h"

#include "../node.h"

#include "../list.h"

#include "../polygon.h"

#include "../nodepol.h"

#include "../nodeint.h"

#include "../map.h"

#include "../math.h"

#include "../mesh.h"

#include "../sector.h"

#include "../nodesec.h"

#include "../refsys.h"

#include "../camsys.h"

#include "../nodesec.h"

#include "../lighttab.h"

#include "../eventtab.h"

#include "../scenery.h"

#include "../object.h"

#include "../nodeobj.h"

#include "nodecomp.h"

#include "cocamsys.h"

#include "comesh.h"

#include "cokbcont.h"



/* constructor - set the type variable from CNodeComponent
**********************************************************/
CComponentKbControl::CComponentKbControl ()
{
   m_dwMyTypeID = TYPE_KBCONTROL;
}



/* Move the object between sectors.
**********************************************************/
void CComponentKbControl::mChangeSector ( CNodeSector* poDestSector )
{
   // remove the object from the sector
   CNode
      *poMyNode = m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetValue ( m_poMyObj->m_dwMyID );
   m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRemoveKey ( m_poMyObj->m_dwMyID, FALSE );

   // add references to new sector
   poDestSector->m_oSector.m_oObjects.mAddKey ( m_poMyObj->m_dwMyID, (CNode*) poMyNode );

   // change my sector
   m_poMyObj->m_poMySec = poDestSector;
}



/* Run the camera.
pData1 - not used
pData2 - not used
pData3 - not used
Return value 0 - ok / 1 - user press exit command
**********************************************************/
PTRVAL CComponentKbControl::mActivate ( PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 )
{
   pData1 = pData1;  // to avoid GCC warnings
   pData2 = pData2;  // to avoid GCC warnings
   pData3 = pData3;  // to avoid GCC warnings

   // get the reference system
   CComponentCameraSystem
      *poComp = (CComponentCameraSystem*) m_poMyObj->mRetComponent ( SLOT_REFSYS );
   if ( poComp == NULL )
      ABORT ( MSG_ERROR_NO_TYPE, __FILE__, __LINE__ );
   CCameraSystem
      *poRefSys = &poComp->m_oCamSys;

   // define rotation angles (between 0 and 255)
   BYTE
      yAX = 0,
      yAY = 0,
      yAZ = 0;
   
   // move direction
   tagfXYZ
      stDirection;
   stDirection.fX = 0;
   stDirection.fY = 0;
   stDirection.fZ = 0;

   // keyboard input

   // verify keyboard for Linux SVGALib target

   #ifdef TRG_LINUX   
   switch ( vga_getkey () ) {
      case 27:   // ESC
         // verify if need exit (!=91) or is a special key (==91)
         if ( vga_getkey () != 91 ) {
            m_poMyObj->m_poScen->m_bEnd = TRUE;
            return 1;
         }
         else
            switch ( vga_getkey () ) {
               case 'A':      // key UP
                  stDirection.fX += poRefSys->m_stJ.fX * poRefSys->m_fSpeed;
                  stDirection.fY += poRefSys->m_stJ.fY * poRefSys->m_fSpeed;
                  stDirection.fZ += poRefSys->m_stJ.fZ * poRefSys->m_fSpeed;
                  break;
               case 'B':      // key DOWN
                  stDirection.fX -= poRefSys->m_stJ.fX * poRefSys->m_fSpeed;
                  stDirection.fY -= poRefSys->m_stJ.fY * poRefSys->m_fSpeed;
                  stDirection.fZ -= poRefSys->m_stJ.fZ * poRefSys->m_fSpeed;
                  break;
               case 'C':      // key RIGHT
                  stDirection.fX += poRefSys->m_stI.fX * poRefSys->m_fSpeed;
                  stDirection.fY += poRefSys->m_stI.fY * poRefSys->m_fSpeed;
                  stDirection.fZ += poRefSys->m_stI.fZ * poRefSys->m_fSpeed;
                  break;
               case 'D':      // key LEFT
                  stDirection.fX -= poRefSys->m_stI.fX * poRefSys->m_fSpeed;
                  stDirection.fY -= poRefSys->m_stI.fY * poRefSys->m_fSpeed;
                  stDirection.fZ -= poRefSys->m_stI.fZ * poRefSys->m_fSpeed;
                  break;
            }
            break;

      // forward
      case 'A':
      case 'a':
         stDirection.fX += poRefSys->m_stK.fX * poRefSys->m_fSpeed;
         stDirection.fY += poRefSys->m_stK.fY * poRefSys->m_fSpeed;
         stDirection.fZ += poRefSys->m_stK.fZ * poRefSys->m_fSpeed;
         break;

      // backward
      case 'Z':
      case 'z':
         stDirection.fX -= poRefSys->m_stK.fX * poRefSys->m_fSpeed;
         stDirection.fY -= poRefSys->m_stK.fY * poRefSys->m_fSpeed;
         stDirection.fZ -= poRefSys->m_stK.fZ * poRefSys->m_fSpeed;
         break;

      // ROTATE
      case '8':       // key UP
         yAX -= 1;
         break;
      case '2':       // key DOWN
         yAX += 1;
         break;
      case '6':
         yAY -= 1;
         break;
      case '4':
         yAY += 1;
         break;
      case '7':
         yAZ += 1;
         break;
      case '9':
         yAZ -= 1;
         break;
   }

   #endif // END LINSVGA CODE

   // verify keyboard for DJGPP target
   #ifdef TRG_DJGPP
   
   // test exit
   if ( m_oKb.mTestKeyPress ( K_ESC ) ) {
      m_poMyObj->m_poScen->m_bEnd = TRUE;
      return 1;
   }
   // test ALT
   if ( m_oKb.mTestKeyPress ( K_ALT ) || m_oKb.mTestKeyPress ( K_ALT | SPECIAL ) ) {
      // move
      if ( m_oKb.mTestKeyPress ( K_UP ) || m_oKb.mTestKeyPress ( K_UP | SPECIAL ) ) {
         stDirection.fX += poRefSys->m_stJ.fX * poRefSys->m_fSpeed;
         stDirection.fY += poRefSys->m_stJ.fY * poRefSys->m_fSpeed;
         stDirection.fZ += poRefSys->m_stJ.fZ * poRefSys->m_fSpeed;
      }
      if ( m_oKb.mTestKeyPress ( K_DOWN ) || m_oKb.mTestKeyPress ( K_DOWN | SPECIAL ) ) {
         stDirection.fX -= poRefSys->m_stJ.fX * poRefSys->m_fSpeed;
         stDirection.fY -= poRefSys->m_stJ.fY * poRefSys->m_fSpeed;
         stDirection.fZ -= poRefSys->m_stJ.fZ * poRefSys->m_fSpeed;
      }
      if ( m_oKb.mTestKeyPress ( K_RIGHT ) || m_oKb.mTestKeyPress ( K_RIGHT | SPECIAL ) ) {
         stDirection.fX += poRefSys->m_stI.fX * poRefSys->m_fSpeed;
         stDirection.fY += poRefSys->m_stI.fY * poRefSys->m_fSpeed;
         stDirection.fZ += poRefSys->m_stI.fZ * poRefSys->m_fSpeed;
      }
      if ( m_oKb.mTestKeyPress ( K_LEFT ) || m_oKb.mTestKeyPress ( K_LEFT | SPECIAL ) ) {
         stDirection.fX -= poRefSys->m_stI.fX * poRefSys->m_fSpeed;
         stDirection.fY -= poRefSys->m_stI.fY * poRefSys->m_fSpeed;
         stDirection.fZ -= poRefSys->m_stI.fZ * poRefSys->m_fSpeed;
      }
   }
   else {
      // rotations
      if ( m_oKb.mTestKeyPress ( K_UP ) || m_oKb.mTestKeyPress ( K_UP | SPECIAL ) )
         yAX -= 1;
      if ( m_oKb.mTestKeyPress ( K_DOWN ) || m_oKb.mTestKeyPress ( K_DOWN | SPECIAL ) )
         yAX += 1;
      if ( m_oKb.mTestKeyPress ( K_RIGHT ) || m_oKb.mTestKeyPress ( K_RIGHT | SPECIAL ) )
         yAY -= 1;
      if ( m_oKb.mTestKeyPress ( K_LEFT ) || m_oKb.mTestKeyPress ( K_LEFT | SPECIAL ) )
         yAY += 1;
   }
   if ( m_oKb.mTestKeyPress ( K_HOME ) )
      yAZ += 1;
   if ( m_oKb.mTestKeyPress ( K_PG_UP ) )
      yAZ -= 1;
   if ( m_oKb.mTestKeyPress ( K_A ) ) {
      stDirection.fX += poRefSys->m_stK.fX * poRefSys->m_fSpeed;
      stDirection.fY += poRefSys->m_stK.fY * poRefSys->m_fSpeed;
      stDirection.fZ += poRefSys->m_stK.fZ * poRefSys->m_fSpeed;
   }
   if ( m_oKb.mTestKeyPress ( K_Z ) ) {
      stDirection.fX -= poRefSys->m_stK.fX * poRefSys->m_fSpeed;
      stDirection.fY -= poRefSys->m_stK.fY * poRefSys->m_fSpeed;
      stDirection.fZ -= poRefSys->m_stK.fZ * poRefSys->m_fSpeed;
   }
   
   #endif // END DJGPP CODE

   // update camera

   // the new camera position without test
   tagfXYZ
      stPosNew;
   stPosNew.fX = poRefSys->m_stPos.fX + stDirection.fX;
   stPosNew.fY = poRefSys->m_stPos.fY + stDirection.fY;
   stPosNew.fZ = poRefSys->m_stPos.fZ + stDirection.fZ;

   // detect collision between parent mesh and object
   BYTE
      yCode;
   CNodePolygon
      *poPol = m_poMyObj->m_poMySec->m_oSector.mDetectCollision ( stPosNew, poRefSys->m_fRadius, yCode );

   // if not colliding with parent sector test between objects inside sector
   if ( poPol == NULL ) {

      // number of objects inide sector
      DWORD
         dwNum = m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetSize ();

      // for each object
      for ( DWORD dwcInd = 0; dwcInd < dwNum; dwcInd ++ ) {

         // get key (object ID)
         DWORD
            dwKey = m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetKey ( dwcInd );

         // get object pointer
         CNodeObject
            *poObj = (CNodeObject*) m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetValue ( dwKey );

	 CNodeComponent
            *poBody = (CNodeComponent*) poObj->m_oObject.mRetComponent ( SLOT_BODY );

         // verify if has a body, if no skip this object
	 if ( poBody == NULL )
	    continue;
	    
         // verify if body not found, or not compatible type, or
         // is the same object (impossible collide with it self)
         if ( &poObj->m_oObject == m_poMyObj ||
              poBody == NULL ||
              poBody->m_dwMyTypeID != TYPE_MESH )
            continue;
         else
            // make test
            poPol = ( (CComponentMesh*) poBody )->m_oMesh.mDetectCollision ( stPosNew, poRefSys->m_fRadius, yCode );

         // if collide
         if ( poPol != NULL )
            break;
      }
   }

   // if have collided
   if ( poPol != NULL ) {

      // test cases

      // if portal crossing
      if ( yCode & 2 )
         // move the instance to the destin sector
         mChangeSector ( poPol->m_oPolygon.m_stPortal.poSector );

      // if collide ONLY with solid poly
      if ( yCode == 1 ) {
         stDirection.fX += poPol->m_oPolygon.m_stNormal.fX * poRefSys->m_fSpeed;
         stDirection.fY += poPol->m_oPolygon.m_stNormal.fY * poRefSys->m_fSpeed;
         stDirection.fZ += poPol->m_oPolygon.m_stNormal.fZ * poRefSys->m_fSpeed;
      }
   }

   // change position
   poRefSys->m_stPos.fX += stDirection.fX;
   poRefSys->m_stPos.fY += stDirection.fY;
   poRefSys->m_stPos.fZ += stDirection.fZ;
   // rotate
   poRefSys->mRotateTransBase ( yAX, yAY, yAZ );

   return 0;
}
