/* Implement a sherical (omni) light source component.

TIPS:
- see the mActivate method to know what parameters can be used. */



#include "../defs.h"

#include "../mem.h"

#include "../array.h"

#include "../string.h"

#include "../file.h"

#include "../filerem.h"

#include "../filepcx.h"

#include "../node.h"

#include "../nodeflo.h"

#include "../list.h"

#include "../polygon.h"

#include "../nodepol.h"

#include "../map.h"

#include "../texture.h"

#include "../lighttab.h"

#include "../nodelii.h"

#include "../nodelgt.h"

#include "../math.h"

#include "../mesh.h"

#include "../sector.h"

#include "../nodesec.h"

#include "../refsys.h"

#include "../camsys.h"

#include "../object.h"

#include "../nodeint.h"

#include "../nodetex.h"

#include "../nodeobj.h"

#include "../nodexyz.h"

#include "../eventtab.h"

#include "../filetxe.h"

#include "../scenery.h"

#include "nodecomp.h"

#include "cosecset.h"

#include "comesh.h"

#include "cocamsys.h"

#include "colsomni.h"



/* contructor - init with default component type
**********************************************************/
CComponentLightSourceOmni::CComponentLightSourceOmni ()
{
   m_dwMyTypeID = TYPE_LIGHT_SOURCE_OMNI;
   m_fLight = DEFAULT_LIGHT_LEVEL;
   m_dwLightID = DEFAULT_LIGHT_TABLE;
   m_fRadius = DEFAULT_RADIUS;
   m_poMesh = NULL;
}



/* Activate: do something acording the parameter pData1.

pData1 - if 0 - does nothing
            1 - copy data from another omni component to this component
            2 - set light source data
            3 - compute lighting (vertexes light levels) for a given mesh
            4 - compute lighting for the sector where the object is inside
            5 - clear source light level information from vertexes in
                the given mesh
            6 - clear source light level information from vertexes in
                sector where the object is inside

pData2 - if pData1 is 0 - not used
                      1 - point the source CComponentOmni
                      2 - not used
                      3 - point the mesh to be lighted
                      4 - not used
                      5 - point the mesh to be lighted
                      6 - not used

pData3 - if pData1 is 0 - not used
                      1 - not used
                      2 - point a CList (with CNodeFloat) with
                          light parameters: light level, ligth table ID,
                          radius. If the respective element is -1 is used
                          the engine default value.
                      3 - not used
                      4 - not used
                      5 - not used
                      6 - not used

return - if pData1 is 0 - value 0
                      1 - value 0
                      2 - value 0
                      3 - value 0
                      4 - value 0
                      5 - value 0
                      6 - value 0
**********************************************************/
PTRVAL CComponentLightSourceOmni::mActivate ( PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 )
{                                       
   pData3 = pData3;

   // make action acording pData1
   switch ( pData1 ) {

      case 1:
         {
            // the parameters list
            CComponentLightSourceOmni
               *poSrc = (CComponentLightSourceOmni*) pData2;
            // if component doesn't exist
            if ( poSrc == NULL )
               return 0;

            m_fLight    = poSrc->m_fLight;
            m_dwLightID = poSrc->m_dwLightID;
            m_fRadius   = poSrc->m_fRadius;
            return 0;
         }
         break;

      case 2:
         {
            // the parameters list
            CList
               *poList = (CList*) pData3;
            // if list doesn't exist
            if ( poList == NULL )
               return 0;

            // set light level
            if ( ( (CNodeFloat*) poList->mRetNode ( 0 ) )->m_fValue == -1 )
               m_fLight = DEFAULT_LIGHT_LEVEL;
            else
               m_fLight = ( (CNodeFloat*) poList->mRetNode ( 0 ) )->m_fValue;

            // set light table ID
            if ( ( (CNodeFloat*) poList->mRetNode ( 1 ) )->m_fValue == -1 )
               m_dwLightID = DEFAULT_LIGHT_TABLE;
            else
               m_dwLightID = (DWORD) ( (CNodeFloat*) poList->mRetNode ( 1 ) )->m_fValue;

            // set radius
            if ( ( (CNodeFloat*) poList->mRetNode ( 2 ) )->m_fValue == -1 )
               m_fRadius = DEFAULT_RADIUS;
            else
               m_fRadius = ( (CNodeFloat*) poList->mRetNode ( 2 ) )->m_fValue;

            // can't be zero
            if ( m_fRadius == 0 )
               m_fRadius = 0.0001;

            return 0;
         }
         break;

      case 3:
         {
            // the mesh
            CMesh
               *poMesh = (CMesh*) pData2;
            // if mesh doesn't exist
            if ( poMesh == NULL )
               return 0;

            // get this object position
            CNodeComponent
               *poComp = m_poMyObj->mRetComponent ( SLOT_REFSYS );

            if ( poComp == NULL )
               return 0;
            CCameraSystem
               *poCamSys = (CCameraSystem*) &( (CComponentCameraSystem*) poComp )->m_oCamSys;

            // for each vertex in mesh
            poMesh->m_oVertexes.mSelectStart ();
            poMesh->m_oVertLight.mSelectStart ();
            while ( poMesh->m_oVertexes.mRetExistSelection () ) {

               // compute the maximal distance between vertex and light
               float
                  fDist = m_oM.mRetMaximalDistance ( poCamSys->m_stPos, ( (CNodeXYZ*) poMesh->m_oVertexes.mRetSelected () )->m_stXYZ );

               // is is in the range, compute the normalized light level
               if ( fDist < m_fRadius )

                  // add the light level generated from this light
                  // source to other light sources
                  ( (CNodeLightInfo*) poMesh->m_oVertLight.mRetSelected () )->m_stValue.fSource += ( ( m_fRadius - fDist ) / m_fRadius ) * m_fLight;

               // next vertex
               poMesh->m_oVertexes.mSelectNext ();
               poMesh->m_oVertLight.mSelectNext ();
            }

            return 0;
         }
         break;

      case 4:
         {
            // the sector mesh
            CNodeSector
               *poSec = m_poMyObj->m_poMySec;

            // if sector doesn't exist
            if ( poSec == NULL )
               return 0;

            m_poMesh = &poSec->m_oSector;

            return mActivate ( 3, (PTRVAL) (void*) &poSec->m_oSector, 0 );
         }
         break;
   
      case 5:
         {
            // the mesh
            CMesh
               *poMesh = (CMesh*) pData2;
            // if mesh doesn't exist
            if ( poMesh == NULL )
               return 0;

            // for each vertex light info in mesh
            poMesh->m_oVertLight.mSelectStart ();
            while ( poMesh->m_oVertLight.mRetExistSelection () ) {

               ( (CNodeLightInfo*) poMesh->m_oVertLight.mRetSelected () )->m_stValue.fSource = 0;

               // next vertex
               poMesh->m_oVertLight.mSelectNext ();
            }

            return 0;
         }
         break;

      case 6:
         {
            // the sector mesh
            CNodeSector
               *poSec = m_poMyObj->m_poMySec;

            // if sector doesn't exist
            if ( poSec == NULL )
               return 0;

            m_poMesh = &poSec->m_oSector;

            return mActivate ( 5, (PTRVAL) (void*) &poSec->m_oSector, 0 );
         }
         break;

      default:
         return 0;
         break;
   }

   return 0;
}
