/* Declare a CComponentSectorsSet component.
This component is used to store a scenery description like a set
of adjacent sectors. */



#include "../defs.h"
       
#include "../node.h"

#include "../list.h"

#include "../map.h"

#include "../polygon.h"

#include "../nodepol.h"

#include "../nodeint.h"

#include "../nodeflo.h"

#include "../nodexyz.h"

#include "../math.h"

#include "../mesh.h"

#include "../sector.h"

#include "../nodesec.h"

#include "../object.h"

#include "../nodeobj.h"

#include "nodecomp.h"

#include "cosecset.h"



/* contructor - init with default component type
**********************************************************/
CComponentSectorsSet::CComponentSectorsSet ()
{
   m_dwMyTypeID = TYPE_SECTORSSET;
}



/* destructor - desalocate data
**********************************************************/
CComponentSectorsSet::~CComponentSectorsSet ()
{
}



/* Insert a new sector in object. dwSectorID is the sector ID.
To change sector data you must first call CSector::mBeginAddSector.
return a pointer to created sector or NULL if error.
**********************************************************/
CNodeSector* CComponentSectorsSet::mAddSector ( DWORD dwSectorID )
{
   // if key already defined, ID exist, return
   if ( m_oSectors [ dwSectorID ] != NULL ) {
      printf ( MSG_ERROR_SECTOR_ID_ALREADY_DEFINED, m_poMyObj->m_dwMyID, dwSectorID );
      return NULL;
   }

   // try create sector node
   CNodeSector
      *poSector = new CNodeSector;
   if ( poSector == NULL ) {
      printf ( MSG_ERROR_MEMORY_LOW );
      printf ( MSG_DEFAULT_ABORT_INFO, __FILE__, __LINE__ );
      return NULL;
   }

   // add in object
   if ( ! m_oSectors.mAddKey ( dwSectorID, (CNode*) poSector ) ) {
      printf ( MSG_ERROR_MEMORY_LOW );
      printf ( MSG_DEFAULT_ABORT_INFO, __FILE__, __LINE__ );
      return NULL;
   }

   // update setor pointer to this object (NULL if root sector)
   poSector->m_oSector.m_poObject = m_poMyObj;

   return poSector;
}
   


/* Update portal pointers for the sectors.
**********************************************************/
void CComponentSectorsSet::mUpdatePortalsPointers ()
{
   CNodeSector
      *poSec;
   // scan sectors vailable
   DWORD
      dwNumSectors = m_oSectors.mRetSize ();
   for ( DWORD dwcInd = 0; dwcInd < dwNumSectors; dwcInd ++ ) {

      // for each key (sector)
      DWORD
         dwKey = m_oSectors.mRetKey ( dwcInd );
      poSec = (CNodeSector*) m_oSectors.mRetValue ( dwKey );

      poSec->m_oSector.mSetPortalPointers ( &m_oSectors );
   }

}



/* Search one for one object instance in the sectors set.
If not found, in the objects that are inside the sectors.
return - a pointer to the object / NULL - if not found
**********************************************************/
CNodeObject* CComponentSectorsSet::mSearchObject ( DWORD dwObjID )
{
   // scan sectors vailable
   DWORD
      dwNumSectors = m_oSectors.mRetSize ();
   CNodeSector
      *poSec;
   CNodeObject
      *poObj;
   for ( DWORD dwcInd = 0; dwcInd < dwNumSectors; dwcInd ++ ) {

      // for each key (sector)
      DWORD
         dwKey = m_oSectors.mRetKey ( dwcInd );
      poSec = (CNodeSector*) m_oSectors.mRetValue ( dwKey );

      // verify if found (search return not NULL) else continue search
      if ( ( poObj = (CNodeObject*) poSec->m_oSector.mSearchObject ( dwObjID ) ) != NULL )
         // return the object
         return poObj;         
   }

   // if not found
   return NULL;
}



/* Activate: do something acording the parameter pData1.
pData1 - if 0 - does nothing
            1 - make a copy of the sectors pointed in pData2
            2 - add a new sector
            3 - update the portals pointers
            4 - search for one object instance inside all sectors
            5 - return a given sector
pData2 - if pData1 is 0 - not used
                      1 - point a CMap object with source sectors
                      2 - has the ID of the sector to be created
                      3 - not used
                      4 - the object ID to search
                      5 - the sector ID to return
pData3 - not used
return - if pData1 is 0 - value 0
                      1 - value 0
                      2 - a pointer to a CNodeSector with the new sector
                      3 - value 0
                      4 - object pointer / NULL - if object not found
                      5 - sector pointer / NULL - if sector not found
**********************************************************/
PTRVAL CComponentSectorsSet::mActivate ( PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 )
{
   pData3 = pData3;
   
   // make action acording pData1
   switch ( pData1 ) {

      case 1:
         {
            // the source sectors
            CMap
               *poSectors = (CMap*) pData2;

            // test errors
            if ( (void*) pData2 == NULL )
               return 0;

            // copy the sectors
            // for each sector in source
            for ( DWORD dwcS = 0; dwcS < poSectors->mRetSize (); dwcS ++ ) {

               // get the source sector ID
               DWORD
                  dwSecID = poSectors->mRetKey ( dwcS );
               // get the source sector
               CNodeSector
                  *poSrcSec = (CNodeSector*) poSectors->mRetValue ( dwSecID );

               // create and add sector
               CNodeSector
                  *poNewSec = (CNodeSector*) mActivate ( 2, dwSecID, 0 );

               // add mesh to sector
               poNewSec->m_oSector.mBeginAddSector ();
               poNewSec->m_oSector.mAppendObj ( poSrcSec->m_oSector );
               poNewSec->m_oSector.mEndAddSector ();
            }
         }
         break;

      case 2:
         return (PTRVAL) mAddSector ( pData2 );
         break;

      case 3:
         mUpdatePortalsPointers ();
         break;

      case 4:
         return (PTRVAL) mSearchObject ( pData2 );
         break;

      case 5:
         return (PTRVAL) m_oSectors [ pData2 ];
         break;

      default:
         return 0;
         break;
   }

   return 0;
}

