/* Declare draw.cc */



// verify if has been declared, if no do it
#ifndef __3DFC_DRAW_H__



// signals file inclusion
#define __3DFC_DRAW_H__



class CDraw {

public:

   CDraw ();
   CDraw ( CLightTable& oLgT, INT nResX, INT nResY, CMem& oFrBuff );
   ~CDraw ();

protected:

   // information for one side of the polygon edges per each scanline
   // (used for scan conversion forward and backward)
   struct tagPolyEdge {

      INT   nX;             // the x's of the polygon edge, (scanline X coords)
      INT   nColor;         // the color or light table index (used for gouraud polys)
      tagXY stTex;          // the texture texels coords
   };

   INT m_nSize;                 // frame buffer size in bytes (resX * resY)
   CMem* m_poFrBuff;            // frame buffer pointer

   // poly scan conversion
   tagPolyEdge* m_pastForwardEdges;   // polygon edges data in forward scan
   tagPolyEdge* m_pastBackwardEdges;  // polygon edges data in backward scan

   // poly edges functions
   void mScanFlatEdge ( tagPolyEdge* pastEdge, tagXY& stV1, tagXY& stV2 );
   void mScanIntLgtEdge ( tagPolyEdge* pastEdge,
                           tagXY& stV1, tagXY& stV2,
                           BYTE yColor1, BYTE yColor2 );
   void mScanTexturedEdge ( tagPolyEdge* pastEdge,
                            tagXY& stP1, tagXY& stP2,
                            tagXY& stT1,  tagXY& stT2 );
   void mScanIntLgtTexturedEdge ( tagPolyEdge* pastEdge,
                                   tagXY& stP1,  tagXY& stP2,
                                   tagXY& stT1,  tagXY& stT2,
                                   float& fL1,   float& fL2 );
   
   // poly scan conversion functions
   void mHorizontalLine ( tagPolyEdge& stEdgeIni, tagPolyEdge& stEdgeEnd,
                          UINT uCol, PTRVAL pLineAddress );
   void mHorizontalIntLgtLine ( tagPolyEdge& stEdgeIni,
                                 tagPolyEdge& stEdgeEnd,
                                 PTRVAL pLineAddress );
   void mHorizontalTexturedLine ( tagPolyEdge& stEdgeIni,
                                  tagPolyEdge& stEdgeEnd,
                                  tagInterval& stLimits,
                                  BYTE* pyTex, PTRVAL pLineAddress,
                                  DWORD dwLightLevel );
   void mHorizontalIntLgtTexturedLine ( tagPolyEdge& stEdgeIni,
                                         tagPolyEdge& stEdgeEnd,
                                         tagInterval& stLimits,
                                         BYTE* pyTex, PTRVAL pLineAddress );

public:

   INT m_nResX;          // frame buffer width
   INT m_nResY;          // frame buffer height
   INT m_nMidX;          // frame buffer midle X value (res X / 2)
   INT m_nMidY;          // frame buffer midle Y value (res Y / 2)
   INT m_nMaxX;          // frame buffer maximum X value (res X - 1)
   INT m_nMaxY;          // frame buffer maximum Y value (res Y - 1)

   CLightTable* m_poLightTable; // light table pointer

   // basic functions
   BOOL mCreate ( CLightTable& oLgT, INT nResX, INT nResY, CMem& oFrBuff );
   void mDestroy ();
   void mSetPixel ( INT nX, INT nY, BYTE yCol );
   void mClear ( BYTE yCol );
   void mCopy ( CMem& oFrBuffSource );
   void mCopy ( INT nOX, INT nOY, INT nWidth, INT nHeight,
                CMem& oFrBuffSource );
   void mLine ( INT nXI, INT nYI, INT nXF, INT nYF, BYTE yCol );

   // portal functions (clip)
   void mResetClipEdgeBuffer ( tagInterval* pastEdgeBuffer );
   BOOL mSetClipEdgeBuffer ( tagXY* pastXYZes, INT nNumVertexes,
                             tagInterval* pastEdgeBuffer,
                             tagRectangle& stClipBox );

   // poly draw functions
   void mFlatPolygon4 ( INT nX1, INT nY1, INT nX2, INT nY2,
                        INT nX3, INT nY3, INT nX4, INT nY4, BYTE yCol );
   void mFlatPolygon ( tagXY* pastXYZes, INT nNumVertexes,
                       BYTE yCol );
   void mIntLgtPolygon ( tagXY* pastXYZes,
                          BYTE* payColors, INT nNumVertexes );
   void mTexturedPolygon ( tagXY* pastPolVertexes,
                           tagXY* pastTexVertexes,
                           INT nNumVertexes, CTexture& oTx,
                           tagInterval* pastEdgeBuffer = NULL,
                           float fNormLightLevel = DEFAULT_LIGHT_LEVEL );
   void mIntLgtTexturedPolygon ( tagXY* pastPolVertexes,
                                  tagXY* pastTexVertexes,
                                  float* pafNormLights,
                                  INT nNumVertexes, CTexture& oTx,
                                  tagInterval* pastEdgeBuffer = NULL );
};



// end declarations
#endif
