/* Main engine control module:
 - make message map
 - socket communication
 - call rendering pipeline. */



#include <string.h>

#include <math.h>

#include "defs.h"

#include "mem.h"

#include "array.h"

#include "string.h"

#include "texture.h"

#include "file.h"

#include "filerem.h"

#include "filepcx.h"

#include "lighttab.h"

#include "draw.h"

#include "node.h"

#include "list.h"

#include "polygon.h"

#include "nodepol.h"

#include "socket.h"

#include "svsocket.h"

#include "nodexyz.h"

#include "map.h"

#include "math.h"

#include "mesh.h"

#include "sector.h"

#include "nodesec.h"

#include "refsys.h"

#include "camsys.h"

#include "object.h"

#include "nodeint.h"

#include "nodetex.h"

#include "nodeobj.h"

#include "nodexyz.h"

#include "eventtab.h"

#include "comp/nodecomp.h"

#include "comp/cokbcont.h"

#include "comp/codata.h"

#include "comp/cocamsys.h"

#include "comp/comesh.h"

#include "filetxe.h"

#include "scenery.h"

#include "render.h"

#include "engine.h"



/* constructor - init empty
**********************************************************/
CEngine::CEngine ()
{
   mDestroy ();
}



/* constructor - init system
**********************************************************/
CEngine::CEngine ( CScenery& oScen, INT nResX, INT nResY, CMem& oFrBuff )
{
   mCreate ( oScen, nResX, nResY, oFrBuff );
}



/* destructor
**********************************************************/
CEngine::~CEngine ()
{
   mDestroy ();
}



/* Init system.
oScen define the scenery to render
nResX define the X resolution to frame buffer
nResY define the Y resolution to frame buffer
oFrBuffer define the frame buffer
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CEngine::mCreate ( CScenery& oScen, INT nResX, INT nResY, CMem& oFrBuff )
{
   mDestroy ();

   m_poScen = &oScen; // define scenery

   // flag start programm
   m_poScen->m_bEnd = FALSE;

   if ( m_stSocketInfo.chSocketUse == 'c' )
      if ( ! m_stSocketInfo.oCliSoc.mCreate () )
         ABORT ( MSG_ERROR_SOCKET_CREATION, __FILE__, __LINE__ )
   
   return m_oRender.mCreate ( oScen, nResX, nResY, oFrBuff );
}



/* Destroy fly system.
**********************************************************/
void CEngine::mDestroy ()
{
   m_poScen = NULL;
   m_stSocketInfo.chSocketUse = 'n';
   m_stSocketInfo.bConnected = FALSE;

   m_oRender.mDestroy ();
}



/* Run the objects and create images
return TRUE - ok / FALSE - error or end loop
**********************************************************/
BOOL CEngine::mRun ()
{
   // verify if not have scenery
   if ( m_poScen == NULL )
      return FALSE;

   // run the message system
   m_poScen->m_oEventTab.mSendMessages ();
      
   // socket communication
   
   // verify if is to make socket communication
   if ( m_stSocketInfo.chSocketUse != 'n' )

      // if is the client
      if ( m_stSocketInfo.chSocketUse == 'c' ) {
         
         // make socket communication and update
         // verify if is connected
         if ( m_stSocketInfo.bConnected ) {
    
            // send my position
            CCameraSystem
               *poRefSys = &( (CComponentCameraSystem*) m_poScen->m_poCamera->m_oObject.mRetComponent ( SLOT_REFSYS ) )->m_oCamSys;
            // invert the camera
            m_stSocketInfo.oSocCam.mSetObj ( *poRefSys );
            m_stSocketInfo.oSocCam.mInvertCamera ();
            // copy the camera definition to a buffer to be send
            float
               afCom [ 13 ];
            afCom [ 0 ] = m_stSocketInfo.oSocCam.m_stPos.fX;
            afCom [ 1 ] = m_stSocketInfo.oSocCam.m_stPos.fY;
            afCom [ 2 ] = m_stSocketInfo.oSocCam.m_stPos.fZ;
            afCom [ 3 ] = m_stSocketInfo.oSocCam.m_stI.fX;
            afCom [ 4 ] = m_stSocketInfo.oSocCam.m_stI.fY;
            afCom [ 5 ] = m_stSocketInfo.oSocCam.m_stI.fZ;
            afCom [ 6 ] = m_stSocketInfo.oSocCam.m_stJ.fX;
            afCom [ 7 ] = m_stSocketInfo.oSocCam.m_stJ.fY;
            afCom [ 8 ] = m_stSocketInfo.oSocCam.m_stJ.fZ;
            afCom [ 9 ] = m_stSocketInfo.oSocCam.m_stK.fX;
            afCom [10 ] = m_stSocketInfo.oSocCam.m_stK.fY;
            afCom [11 ] = m_stSocketInfo.oSocCam.m_stK.fZ;
            afCom [12 ] = (float) m_poScen->m_poCamera->m_oObject.m_dwMyID; // send my ID
            // send
            m_stSocketInfo.oCliSoc.mSend ( (void*) afCom, sizeof afCom );

            // read from socket the other camera position
            INT
               nCode;
            // if read some data
            if ( m_stSocketInfo.oCliSoc.mReceive ( (void*) afCom, sizeof afCom, nCode ) ) {

               // verify if connection ended or error
               if ( nCode <= 0 ) {
                  m_stSocketInfo.oCliSoc.mDestroy ();
                  m_stSocketInfo.bConnected = FALSE;
               }
               else { // if connection ok

                  // change the other camera position
                  DWORD
                     dwOtherID = (DWORD) afCom [ 12 ];
                  // search the object
                  CNodeObject
                     *poOther = (CNodeObject*) m_poScen->m_oRoot.m_oSector.mSearchObject ( dwOtherID );
                  // verify if object exist  // REVIEW
                  if ( poOther != NULL ) {
                     CCameraSystem
                        *poRefSys = &( (CComponentCameraSystem*) poOther->m_oObject.mRetComponent ( SLOT_REFSYS ) )->m_oCamSys;
                     // copy the given definition
                     poRefSys->m_stPos.fX = afCom [ 0 ];
                     poRefSys->m_stPos.fY = afCom [ 1 ];
                     poRefSys->m_stPos.fZ = afCom [ 2 ];
                     // REVIEW (no servidor tambem!!!)
                     /*poRefSys->m_stI.fX = afCom [ 3 ];
                     poRefSys->m_stI.fY = afCom [ 4 ];
                     poRefSys->m_stI.fZ = afCom [ 5 ];
                     poRefSys->m_stJ.fX = afCom [ 6 ];
                     poRefSys->m_stJ.fY = afCom [ 7 ];
                     poRefSys->m_stJ.fZ = afCom [ 8 ];
                     poRefSys->m_stK.fX = afCom [ 9 ];
                     poRefSys->m_stK.fY = afCom [10 ];
                     poRefSys->m_stK.fZ = afCom [11 ];*/
                  }
               }  // END IF connection ok
            }
         }
      }
      else {   // if this is the server
         
         // make socket communication and update
         // verify if is connected -> make communication
         if ( m_stSocketInfo.bConnected ) {
            
            // send my position
            CCameraSystem
               *poRefSys = &( (CComponentCameraSystem*) m_poScen->m_poCamera->m_oObject.mRetComponent ( SLOT_REFSYS ) )->m_oCamSys;
            // invert the camera
            m_stSocketInfo.oSocCam.mSetObj (  *poRefSys );
            m_stSocketInfo.oSocCam.mInvertCamera ();
            // copy the camera definition to a buffer to be send
            float
               afCom [ 13 ];
            afCom [ 0 ] = m_stSocketInfo.oSocCam.m_stPos.fX;
            afCom [ 1 ] = m_stSocketInfo.oSocCam.m_stPos.fY;
            afCom [ 2 ] = m_stSocketInfo.oSocCam.m_stPos.fZ;
            afCom [ 3 ] = m_stSocketInfo.oSocCam.m_stI.fX;
            afCom [ 4 ] = m_stSocketInfo.oSocCam.m_stI.fY;
            afCom [ 5 ] = m_stSocketInfo.oSocCam.m_stI.fZ;
            afCom [ 6 ] = m_stSocketInfo.oSocCam.m_stJ.fX;
            afCom [ 7 ] = m_stSocketInfo.oSocCam.m_stJ.fY;
            afCom [ 8 ] = m_stSocketInfo.oSocCam.m_stJ.fZ;
            afCom [ 9 ] = m_stSocketInfo.oSocCam.m_stK.fX;
            afCom [10 ] = m_stSocketInfo.oSocCam.m_stK.fY;
            afCom [11 ] = m_stSocketInfo.oSocCam.m_stK.fZ;
            afCom [12 ] = (float)  m_poScen->m_poCamera->m_oObject.m_dwMyID; // send my ID
            // send
            m_stSocketInfo.oSrvSoc.mSend ( (void*) afCom, sizeof afCom );

            // read from socket the other camera position
            INT
               nCode;

            // if read some data
            if ( m_stSocketInfo.oSrvSoc.mReceive ( (void*) afCom, sizeof afCom, nCode ) ) {

               // verify if connection ended or error
               if ( nCode <= 0 ) {
                  m_stSocketInfo.oSrvSoc.mDestroy ();
                  m_stSocketInfo.bConnected = FALSE;
               }
               else { // if connection ok

                  // change the other camera position
                  DWORD
                     dwOtherID = (DWORD) afCom [ 12 ];
                  // search the object
                  CNodeObject
                     *poOther = (CNodeObject*) m_poScen->m_oRoot.m_oSector.mSearchObject ( dwOtherID );
                  // verify if object exist  // REVIEW
                  if ( poOther != NULL ) {
                     CCameraSystem
                        *poRefSys = &( (CComponentCameraSystem*) poOther->m_oObject.mRetComponent ( SLOT_REFSYS ) )->m_oCamSys;
                     // copy the given definition
                     poRefSys->m_stPos.fX = afCom [ 0 ];
                     poRefSys->m_stPos.fY = afCom [ 1 ];
                     poRefSys->m_stPos.fZ = afCom [ 2 ];
                     // REVIEW (no servidor tambem!!!)
                     /*poRefSys->m_stI.fX = afCom [ 3 ];
                     poRefSys->m_stI.fY = afCom [ 4 ];
                     poRefSys->m_stI.fZ = afCom [ 5 ];
                     poRefSys->m_stJ.fX = afCom [ 6 ];
                     poRefSys->m_stJ.fY = afCom [ 7 ];
                     poRefSys->m_stJ.fZ = afCom [ 8 ];
                     poRefSys->m_stK.fX = afCom [ 9 ];
                     poRefSys->m_stK.fY = afCom [10 ];
                     poRefSys->m_stK.fZ = afCom [11 ];*/
                  }
               }
            }
         }
      }   // END IF server

   // make image
   m_oRender.mRenderFrame ( m_poScen->m_poCamera );

   return ! m_poScen->m_bEnd;
}
