/* Implements a event manager. Create, destroy and generate messages to
objects in scenery.
*/



#include <string.h>

#include "defs.h"

#include "mem.h"

#include "array.h"

#include "string.h"

#include "file.h"

#include "filerem.h"

#include "filepcx.h"

#include "node.h"

#include "nodeflo.h"

#include "list.h"

#include "polygon.h"

#include "nodepol.h"

#include "map.h"

#include "texture.h"

#include "lighttab.h"

#include "nodelgt.h"

#include "math.h"

#include "mesh.h"

#include "sector.h"

#include "nodesec.h"

#include "refsys.h"

#include "camsys.h"

#include "object.h"

#include "nodeint.h"

#include "nodetex.h"

#include "nodeobj.h"

#include "nodexyz.h"

#include "eventtab.h"

#include "comp/nodecomp.h"

#include "comp/cocamsys.h"

#include "comp/comesh.h"

#include "comp/cosecset.h"

#include "filetxe.h"

#include "scenery.h"



/* constructor
**********************************************************/
CEventTable::CEventTable ()
{
   m_pastMessages = NULL;
   m_dwSize = 0;
}



/* destructor - desalocate buffer, if alocated
**********************************************************/
CEventTable::~CEventTable ()
{
   mDestroy ();
}



/* Desalocate - if alocated
**********************************************************/
void CEventTable::mDestroy ()
{
   if ( m_pastMessages == NULL ) {
      delete m_pastMessages;
      m_pastMessages = NULL;
      m_dwSize = 0;
   }
}

      

/* Realocate array, copy the original elements to the new array.
dwSize - size of the new array.
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CEventTable::mRealocate ( DWORD dwSize )
{
   // Create new memory array
   // try alocation
   tagMessage
      *paArray;
   if ( ( paArray = new tagMessage [ dwSize ] ) == NULL ) {
      printf ( MSG_ERROR_MEMORY_LOW );
      printf ( MSG_DEFAULT_ABORT_INFO, __FILE__, __LINE__ );
      return FALSE;
   }

   // copy elements to new array
   // if new array is short than the actual: truncate
   if ( dwSize < m_dwSize )
      memcpy ( (void*) paArray, (void*) m_pastMessages, dwSize * sizeof ( tagMessage ) );
   else
      // if new array has igual size or greater size: copy all
      memcpy ( (void*) paArray, (void*) m_pastMessages, m_dwSize * sizeof ( tagMessage ) );
             
   // desalocate current array
   if ( m_pastMessages != NULL )
      delete m_pastMessages;

   // change current array to new
   m_pastMessages = paArray;
   m_dwSize = dwSize;

   return TRUE;
}



/* Add one message to the Event system
oScenery - scenery, used to find an object using the ID
poObj - the destin object (to send message), if NULL is used dwObjID
        to define the destin
dwObjID - destin object ID, optional if gived a valid poObj
dwSlotID - the slot to be activated (this is the message code)
pData1, pData2, pData3 - message parameters to object
return the index where the element is inserted (message identifier) or
       -1 if error
**********************************************************/
INT CEventTable::mAddMessage ( CScenery& oScenery, CNodeObject* poObj, DWORD dwObjID, DWORD dwSlotID, PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 )
{
   // search for one free element
   DWORD
      dwFree = (DWORD) -1;   // will has index for the free element
   // scan array of messages
   for ( DWORD dwcE = 0; dwcE < m_dwSize; dwcE ++ )
      
      // verify if element has no destin object pointer
      if ( m_pastMessages [ dwcE ].poObj == NULL ) {
         dwFree = dwcE;
         break;
      }

   // verify if not found one free element, if no,
   // grow message array with more 10 elements
   if ( dwFree == (DWORD) -1 ) {
      
      if ( ! mRealocate ( m_dwSize + 10 ) )
         return -1;

      // set array elements with initial data (empty messages)
      for ( DWORD dwcNew = m_dwSize - 9; dwcNew < m_dwSize; dwcNew ++ )
          m_pastMessages [ dwcNew ].poObj = NULL;

      // set where to add the new element
      dwFree = m_dwSize - 10;
   }

   // if need search for the object (pointer is NULL, use the ID)
   if ( poObj == NULL ) {
      m_pastMessages [ dwFree ].poObj = oScenery.m_oRoot.m_oSector.mSearchObject ( dwObjID );
      // verify if found
      if ( m_pastMessages [ dwFree ].poObj == NULL )
         return -1;
   }

   // update message parameters
   m_pastMessages [ dwFree ].dwSlotID = dwSlotID;
   m_pastMessages [ dwFree ].pData1 = pData1;
   m_pastMessages [ dwFree ].pData2 = pData2;
   m_pastMessages [ dwFree ].pData3 = pData3;

   return dwFree;
}



/* Remove the message for the array.
dwPos - the message position in array, identifier
**********************************************************/
void CEventTable::mRemoveMessage ( DWORD dwPos )
{
   // verify if is out off range
   if ( dwPos >= m_dwSize )
      return;

   // set the element with no destin
   m_pastMessages [ dwPos ].poObj = NULL;
}



/* Generate, send, the messages
**********************************************************/
void CEventTable::mSendMessages ()
{
   // scan message array and search for messages to send
   tagMessage
      *pstCursor = m_pastMessages; // scan the message array
   // for each message
   for ( DWORD dwcM = 0; dwcM < m_dwSize; dwcM ++ ) {

      // if has message
      if ( pstCursor->poObj != NULL )
         
         // activate the respective object component
         pstCursor->poObj->m_oObject.mActivateComponent (
                                                pstCursor->dwSlotID,
                                                pstCursor->pData1,
                                                pstCursor->pData2,
                                                pstCursor->pData3 );

      // if this is not the last element
      if ( dwcM < m_dwSize - 1 )
         /// goto next message
         pstCursor ++;
   }
}
