/* File access functions. Enable read, create, write,... text and binary
 files.
A string returned by mRetStr is called token. This string must be beathen
ENTER, SPACE and/or TAB characters. This function work similar as fscanf.

TIPS:
- to know if a file is opened do comparison: CFile::mRetFilePrt () != NULL
- to know if a file exist use CFile::mRetExistFile, pass file name in
  parameters
- in text files separe token with SPACE or ENTER or TAB.
- possible access modes are C standard codes: r=read, w=write, t=text, ...
- not all C file I/O functions are implemented, use CFile::mRetFilePrt ()
  to return the file structure an then use the C standard funtions, as you need.

LIMITS:
- max token size: 300 bytes

BUGS:
- if you use the change file pointers functions the text functions will
  not work correctly in all cases for obviuosly reasons.
- file errors not tested. Ex.: open a file that does not exist cause
  programm exit.
*/



#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "defs.h"

#ifdef TRG_DJGPP
   #include <io.h>
   #include <dir.h>
#endif

#include "mem.h"

#include "string.h"

#include "file.h"



/* constructor - no name, no file open
**********************************************************/
CFile::CFile ()
{
   m_pfFile = NULL;
}



/* constructor - open file, if fail abort program
**********************************************************/
CFile::CFile ( CString oszFile, CString oszMode )
{
   m_pfFile = NULL;

   // try open file
   if ( ! mOpen ( oszFile, oszMode ) )
      ABORT ( MSG_ERROR_FILE_NOT_FOUND, __FILE__, __LINE__ );
}



/* destructor - close file
**********************************************************/
CFile::~CFile ()
{
   mClose ();
}



/* Open (create file if necessary)
oszMode - define mode (example: rt, wb).
return TRUE - file open / FALSE - not open
**********************************************************/
BOOL CFile::mOpen ( CString oszFile, CString oszMode )
{
   // close opened files
   mClose ();

   m_oszFile = oszFile;

   // try open file
   if ( ( m_pfFile = fopen ( (char*) m_oszFile, (char*) oszMode ) ) == NULL )
      return FALSE;

   return TRUE;
}



/* Close file if is opened
**********************************************************/
void CFile::mClose ()
{
   // verify if file is still opened
   if ( m_pfFile != NULL ) {
      fclose ( m_pfFile );
      m_pfFile = NULL;
      m_oszFile.mDestroy ();
   }
}



/* Verify if the specified file exist
return TRUE - yes / FALSE - no
**********************************************************/
BOOL CFile::mRetExistFile ( CString oszFile )
{
   FILE
      *pfFile;

   // try opned file, if fail (NULL) return FALSE
   if ( ( pfFile = fopen ( (char*) oszFile, "rb" ) ) == NULL )
      return FALSE;

   // close file
   fclose ( pfFile );

   return TRUE;
}



/* Return file size, in bytes
DJGPP only.
**********************************************************/
DWORD CFile::mRetSize ()
{
   #ifdef TRG_DJGPP
   return ( filelength ( fileno ( m_pfFile ) ) );
   // fileno ( m_pfFile ) -> return the FILE* handle
   #endif
}



/* Return the file name and extension (no path)
DJGPP only.
**********************************************************/
void CFile::mRetNameExt ( CString& oszName, CString& oszExt )
{
   #ifdef TRG_DJGPP
   char
      achDrive [ MAXDRIVE ],
      achPath [ MAXDIR ],
      achFile [ MAXFILE ],
      achExt [ MAXEXT ];

   fnsplit ( (char*) m_oszFile, achDrive, achPath, achFile, achExt );

   // set file name and ext
   oszName = achFile;
   oszExt = achExt;
   #endif
}



/* Return current file control structure
**********************************************************/
struct FILE* CFile::mRetFilePrt ()
{
   return m_pfFile;
}



/* Read token. If bAbort == TRUE abort program even an error apear
Default bAbort = FALSE.
If bAbort = FALSE: when an error ocurr function return FALSE else TRUE
**********************************************************/
BOOL CFile::mRetStr ( CString& oszStr, BOOL bAbort )
{
   char
      szTk [ 300 ];

   szTk [ 0 ] = 0;

   // try read, if an error ocurr do something
   if ( fscanf ( m_pfFile, "%s", szTk ) == 0 )

      if ( bAbort )
         ABORT ( MSG_ERROR_READING_FILE, __FILE__, __LINE__ )
      else
         return FALSE;

   oszStr = szTk;

   return TRUE;
}



/* Read a token and convert to DWORD, return value.
**********************************************************/
DWORD CFile::mRetDWord ()
{
   CString
      oszStr;

   mRetStr ( oszStr, FALSE );

   return (DWORD) atoi ( (char*) oszStr );
}



/* Read a line. nMaxLenght define the maximum number of chars
to read. return TRUE - read / FALSE - not read, maybe end of file
**********************************************************/
BOOL CFile::mRetLine ( CString& oszStr, DWORD dwMaxLenght )
{
   oszStr.mCreate ( dwMaxLenght );

   if ( fgets ( (char*) oszStr, dwMaxLenght, m_pfFile ) == NULL )
      return FALSE;

   return TRUE;
}



/* 1 - Read an array of dwMax bytes.
return TRUE - read all bytes / FALSE - read not all, maybe end of file
**********************************************************/
BOOL CFile::mRetMem ( DWORD dwMax, void* pvMem )
{
   if ( fread ( pvMem, 1, dwMax, m_pfFile ) < dwMax )
      return FALSE;

   return TRUE;
}



/* 2 - Read an array of dwMax bytes. Put in an CMem object
return TRUE - read all bytes / FALSE - read not all, maybe end of file
**********************************************************/
BOOL CFile::mRetMem ( DWORD dwMax, CMem& oMem )
{
   if ( fread ( (void*) oMem, 1, dwMax, m_pfFile ) < dwMax )
      return FALSE;

   return TRUE;
}



/* Write string. return TRUE - write; FALSE - no
**********************************************************/
BOOL CFile::mSetStr ( CString oszStr )
{
   // try write
   if ( fprintf ( m_pfFile, "%s", (char*) oszStr ) == EOF )
      return FALSE;

   return TRUE;
}



/* Write a DWORD as string. If bSigned == TRUE write an int 32 bits
(with + or -), else write a dword.
return TRUE - write / FALSE - no. Default bSigned == TRUE
**********************************************************/
BOOL CFile::mSetDWord ( DWORD dwN, BOOL bSigned )
{
   char
      szS [ 20 ];

   // verify if must write with sign
   if ( bSigned == TRUE )
      sprintf ( szS, "%+06d", dwN );
   else
      sprintf ( szS, "%05d", dwN );

   CString
      oszStr = szS;

   return mSetStr ( oszStr );
}



/* Write a line in file, insert line-break at end of line.
**********************************************************/
void CFile::mSetLine ( CString oszStr )
{
   fputs ( (char*) oszStr, m_pfFile );
}



/* 1 - Write an array of dwMax bytes.
dwMax - numero de bytes a serem gravados
return - TRUE write, FALSE - not all data writen
**********************************************************/
BOOL CFile::mSetMem ( DWORD dwMax, void* pvMem )
{
   if ( fwrite ( pvMem, dwMax, 1, m_pfFile ) < 1 )
      return FALSE;

   return TRUE;
}



/* 2 - Write an array. oMem object to be writen.
return - TRUE write, FALSE - not all data writen
**********************************************************/
BOOL CFile::mSetMem ( CMem& oMem )
{
   if ( fwrite ( (void*) oMem, oMem.m_dwSize, 1, m_pfFile ) < 1 )
      return FALSE;

   return TRUE;
}



/* Read file searching for a specific sub-string oszStr.
When found return with file pointer pointing next token.
return - TRUE found; FALSE - not
**********************************************************/
BOOL CFile::mSearchStr ( CString oszStr )
{
   CString
      oszStrReaded;

   // read file while tokens readed are not the same as oszStr
   do
      // verify if not found the end of file
      if ( mRetStr ( oszStrReaded, FALSE ) == FALSE )
         return FALSE;

   while ( strcmp ( (char*) oszStrReaded, (char*) oszStr ) != 0 );

   return TRUE;
}



/* Position file pointer in start with offset dwOffset.
**********************************************************/
void CFile::mPosStart ( DWORD dwOffset )
{
   fseek ( m_pfFile, dwOffset, SEEK_SET );
}



/* Position file pointer in current position with offset nOffset.
**********************************************************/
void CFile::mPosOffset ( INT nOffset )
{
   fseek ( m_pfFile, nOffset, SEEK_CUR );
}



/* Position file pointer in end with offset dwOffset to start.
**********************************************************/
void CFile::mPosEnd ( INT nOffset )
{
   fseek ( m_pfFile, - nOffset, SEEK_END );
}
