/* Simple file access functions and text files with remarks.
This class is derived from CFile and have all it's funcionality. The new
feature is only that the metod CFileRem::mRetStr () read tokens until
find a open remark token (/*). When found jump the remark area without
user know and continue seaching next token after remark area,ignoring
remark.

FEATURES:
- read text files with remark: open remark /* and close remark (I can't
  write it here but it's the C close remark). This remark simbols are tokens
  and ALL tokens must be separed with TAB or ENTER or SPACE!
- the remarks can be one inside the other like this, with parentesis:
  ( ( a ) )

TIPS:
- this class derive from CFile, see CFile for more info

BUGS:
- when reading a file with remarks the use of other functions will cause
  confusion in file control and remarks control will not work correctly.
*/



#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "defs.h"

#include "mem.h"

#include "string.h"

#include "file.h"

#include "filerem.h"



/* constructor - define no remarks opened
**********************************************************/
CFileRem::CFileRem ()
{
   m_nRemarkLevel = 0;
}



/* constructor - define no remarks opened,
try open file if fail abort.
**********************************************************/
CFileRem::CFileRem ( CString oszFile, CString oszMode )
{
   m_pfFile = NULL;
   m_nRemarkLevel = 0;
   // try open file
   if ( ! CFile::mOpen ( oszFile, oszMode ) )
      ABORT ( MSG_ERROR_FILE_NOT_FOUND, __FILE__, __LINE__ );
}



/* destructor - close file
**********************************************************/
CFileRem::~CFileRem ()
{
   mClose ();
}



/* Open (create file if necessary)
oszMode - define mode (example: rt, wb).
return TRUE - file open / FALSE - not open
**********************************************************/
BOOL CFileRem::mOpen ( CString oszFile, CString oszMode )
{
   // close opened files
   mClose ();

   return CFile::mOpen ( oszFile, oszMode );
}



/* Set remark level to 0
**********************************************************/
void CFileRem::mClose ()
{
   m_nRemarkLevel = 0;

   CFile::mClose ();
}



/* Read token and ignore remark ares.
If bAbort == TRUE abort program even an error ocurr.
If bAbort = FALSE: when file IO error function return FALSE else TRUE
Default bAbort = FALSE.
**********************************************************/
BOOL CFileRem::mRetStr ( CString& oszStr, BOOL bAbort )
{
   char
      szTk [ 300 ];

   szTk [ 0 ] = 0;

   BOOL
      bExit;

   // read file until found token not in remark area
   do {

      bExit = TRUE;

      // verify if found end remark simbol (*/), if yes decrement level
      if ( strcmp ( szTk, "*/" ) == 0 )
         m_nRemarkLevel --;

      // try read file, if fail exit this function
      if ( fscanf ( m_pfFile, "%s", szTk ) == 0 ) {

         // verify if remark area is still opened (possible error cause)
         if ( m_nRemarkLevel > 0 && bAbort )
            ABORT ( MSG_ERROR_REMARK_OPENED, __FILE__, __LINE__ )

         // verify if must abort or return
         if ( bAbort )
            ABORT ( MSG_ERROR_READING_TOKEN, __FILE__, __LINE__ )
         else
            return FALSE;
      }

      // verify if found end remark without start
      if ( strcmp ( szTk, "*/" ) == 0 && m_nRemarkLevel == 0 ) {
         bExit = FALSE;
         printf ( MSG_WARN_END_REMARK );
      }

      // verify if found start remark
      if ( strcmp ( szTk, "/*" ) == 0 )
         m_nRemarkLevel ++;

   } while ( m_nRemarkLevel != 0 && bExit );

   oszStr = szTk;

   return TRUE;
}



/* Read a token and convert to INT, return value in nVal.
return TRUE - ok / FALSE - error
If bAbort == TRUE abort program even an error ocurr.
If bAbort = FALSE: when an error ocurr function return FALSE else TRUE
Default bAbort = FALSE.
**********************************************************/
BOOL CFileRem::mRetInt ( INT& nVal, BOOL bAbort )
{
   CString
      oszStr;

   // try read token
   if ( ! mRetStr ( oszStr, bAbort ) )
      return FALSE;

   // convert string to int
   nVal = (INT) atoi ( (char*) oszStr );

   return TRUE;
}
