/* Read a text file that describe scenery elements (objects, sectors, define
textures palettes, ...). The data loaded are stored in a CScenery instance.

The file extension commonly used for this files is TXE (TeXt + Elements).

Use the UNIX file separator (/) to refer files, to make all more
compatible with UNIX and DOS.

See doc/txe-spec.txt for more information.
*/
              


#include <stdlib.h>

#include "defs.h"

#include "mem.h"

#include "array.h"

#include "string.h"

#include "node.h"

#include "list.h"

#include "polygon.h"

#include "nodepol.h"

#include "nodeint.h"

#include "file.h"

#include "filerem.h"

#include "texture.h"

#include "lighttab.h"

#include "filepcx.h"

#include "nodetex.h"

#include "nodeflo.h"

#include "map.h"

#include "math.h"

#include "mesh.h"

#include "sector.h"

#include "nodesec.h"

#include "refsys.h"

#include "camsys.h"

#include "eventtab.h"

#include "scenery.h"

#include "filetxe.h"



/* constructor - define no file to read
**********************************************************/
CFileTXE::CFileTXE ()
{
   m_poScen = NULL;
   mDefineIDNames ();
}



/* constructor - read from file, if fail abort.
**********************************************************/
CFileTXE::CFileTXE ( CString oszFile, CScenery& oScenery )
{
   m_poScen = NULL;
   mDefineIDNames ();

   // try load file
   if ( ! mLoad ( oszFile, oScenery ) )
      ABORT ( MSG_ERROR_FILE_NOT_FOUND, __FILE__, __LINE__ )
}



/* destructor
**********************************************************/
CFileTXE::~CFileTXE ()
{
   m_poScen = NULL;
   mClose ();
}



/* Load data from a TXE file to a instance of CScenery.
oszFile - file name (with path and extension)
oScenery - where store data
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CFileTXE::mLoad ( CString oszFile, CScenery& oScenery )
{
   m_poScen = &oScenery;

   // try open file
   if ( ! mOpen ( oszFile, "rt" ) ) {

      #if COMP_SHOWLOADING >= 1
      printf ( MSG_ERROR_FILE_NOT_FOUND );
      #endif

      return FALSE;
   }

   #if COMP_SHOWLOADING >= 1
   printf ( MSG_READING_TXE, (char*) oszFile );
   #endif

   // read TXE file blocks (see ID values in header file)
   INT
      nReturnCode = mReadBlock ( oszFile, ID_NOBLOCK );

   #if COMP_SHOWLOADING >= 1
   printf ( MSG_END_READING_TXE, (char*) oszFile );
   #endif

   // end load
   mClose ();
   m_poScen = NULL;
   if ( nReturnCode == 1 )
      return TRUE;
   else
      return FALSE;
}



/* Read the beginnig of TXE file blocks and define where to
store data in CScenery object.
oszFile - file beeing read,
nBlockType - type of parent block of this block
return 1 - ok / not 1 - error
**********************************************************/
INT CFileTXE::mReadBlock ( CString oszFile, INT nParentBlock )
{
   // while not end do read
   BOOL
      bEnd = FALSE;
   while ( ! bEnd ) {

      // next token string to be read
      CString
         oszStr;

      // try read token, if error supose end of file
      if ( ! mRetStr ( oszStr ) )
      
         // verify if now read cursor is in top-outside-every-blocks
         if ( nParentBlock == ID_NOBLOCK )
            // if no block it's ok find end-of-file
            break;

         else {  // if inside one block it's wrong find end-of-file
            
            #if COMP_SHOWLOADING >= 1
            printf ( MSG_ERROR_UNSPECTED_EOF, (char*) oszFile, (char*) m_aoIDs [ nParentBlock ] );
            #endif

            return 0;
         }

      // show what has been found
      #if COMP_SHOWLOADING == 2
      printf ( MSG_TOKEN_FOUND, (char*) oszStr, (char*) oszFile );
      #endif

      // define type of block (open block is one possible name!)
      switch ( mClassifyBlock ( oszStr, nParentBlock ) ) {

         case ID_ERROR:
            // REVIEW LINUX problema para achar fim arquivo?
            if ( nParentBlock != ID_NOBLOCK ) {

               #if COMP_SHOWLOADING >= 1
               printf ( MSG_ERROR_BLOCK, (char*) oszStr, (char*) m_aoIDs [ nParentBlock ] );
               #endif

               return 0;
            }
            else
               bEnd = TRUE;
            break;

         case ID_NOBLOCK:

            #if COMP_SHOWLOADING >= 1
            printf ( MSG_ERROR_NOBLOCK, (char*) oszStr, (char*) m_aoIDs [ nParentBlock ] );
            #endif

            return 0;
            break;

         case ID_READ:
            // try read open block, if can't return
            if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

               #if COMP_SHOWLOADING >= 1
               printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_READ );
               #endif

               return 0;
            }

            // read file names and open them (one at a time)
            do {

               // try read token, if error: exit
               if ( ! mTryReadToken ( oszStr, ID_READ, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_FILE_NAME, oszFile, FALSE, FALSE ) )
                  return 0;

               // convert 3DFC file separator '/' to system file seperator
               oszStr.mReplace ( _3DFC_FILESEPARATOR, _SYSTEM_FILESEPARATOR );

               // verify if not found end of block
               if ( oszStr != STR_CLOSEBLOCK ) {

                  // read file
                  // define file name to be readed
                  CString
                     oszPath;
                    // verify if the name readed not have drive-separator
                  if ( ! oszStr.mTestFromLeft ( DRIVESEPARATOR ) ) {
                     // define relative path
                     CString
                        oszNot;
                     oszPath = oszFile;
                     if ( ! oszPath.mSplitFromRight ( _SYSTEM_FILESEPARATOR, oszNot ) )
                        oszPath = "";   // no path
                     else { // add file separator at end of path
                        char
                           szFileSep [ 2 ] = { _SYSTEM_FILESEPARATOR, 0 };  // construct string
                        oszPath += szFileSep;
                     }
                  }
                  // define all file name (with path)
                  CString
                     oszRead;
                  oszRead = oszPath + oszStr;
                  // try load file
                  CFileTXE
                     oFile;
                  INT
                     nErrorCode;
                  if ( ( nErrorCode = oFile.mLoad ( oszRead, *m_poScen ) ) != 1 )
                     return nErrorCode;     // error loading (1 OK, no error)
               }
            } while ( oszStr != STR_CLOSEBLOCK ); // while not close block

            #if COMP_SHOWLOADING >= 1
            printf ( MSG_CONTINUE_READING_TXE, (char*) oszFile );
            #endif

            break;

         case ID_PALETTE:
            {
               // try read palette file name, if error exit
               if ( ! mTryReadToken ( oszStr, ID_PALETTE, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_FILE_NAME, oszFile, FALSE, FALSE ) )
                  return 0;

               // convert 3DFC file separator '/' to system file seperator
               oszStr.mReplace ( _3DFC_FILESEPARATOR, _SYSTEM_FILESEPARATOR );

               // read palette from PCX file
               // define file name to be readed
               CString
                  oszPath;
               // verify if the name readed not have drive-separator
               if ( ! oszStr.mTestFromLeft ( DRIVESEPARATOR ) ) {
                  // define relative path
                  CString
                     oszNot;
                  oszPath = oszFile;
                  if ( ! oszPath.mSplitFromRight ( _SYSTEM_FILESEPARATOR, oszNot ) )
                     oszPath = "";   // no path
                  else { // add file separator at end of path
                     char
                        szFileSep [ 2 ] = { _SYSTEM_FILESEPARATOR, 0 };  // construct string
                     oszPath += szFileSep;
                  }
               }
               // define all file name (with path)
               CString
                  oszRead;
               oszRead = oszPath + oszStr;
               // load palette from file
               CFilePCX
                  oFile;
               CMem
                  oPal;

               #if COMP_SHOWLOADING >= 1
               printf ( MSG_READING_PALETTE, (char*) oszRead );
               #endif

               if ( ! oFile.mLoadPalette ( oszRead, oPal ) ) { // try load

                  #if COMP_SHOWLOADING >= 1
                  printf ( MSG_ERROR_NOT_FOUND, (char*) oszRead );
                  #endif

                  return 0;
               }
               // try add in scenery
               if ( ! m_poScen->mAddPalette ( oPal ) )
                  return 0;
            }

            break;

         case ID_INITMESSAGES:
            {
               // try read open block, if can't return
               if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

                  #if COMP_SHOWLOADING >= 1
                  printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_INITMESSAGES );
                  #endif

                  return 0;
               }

               // load one message
               do {

                  // try object identifier, if error: exit
                  if ( ! mTryReadToken ( oszStr, ID_INITMESSAGES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, FALSE, FALSE ) )
                     return 0;

                  // verify if not read end of block
                  if ( oszStr != STR_CLOSEBLOCK ) {

                     // define ID as a number
                     DWORD
                        dwID = (DWORD) atoi ( (char*) oszStr );
                  
                     // read slot ID, parameters 1 and 2
                     DWORD
                        adwVals [ 3 ];
                     if ( ! mReadArray ( 3, adwVals, ID_INITMESSAGES, oszFile ) )
                        return 0;

                     // try read tuple separator, if can't return
                     // is manatory tuple separator
                     if ( ! mRetStr ( oszStr ) )
                        return 0;
                     if ( oszStr != STR_TUPLESEPARATOR )
                        return 0;

                     // try add in scenery
                     if ( m_poScen->mAddMessage ( NULL, dwID, adwVals [ 0 ],
                                                  adwVals [ 1 ],
                                                  adwVals [ 2 ], 0 ) == -1 )
                        return 0;
                  }
               } while ( oszStr != STR_CLOSEBLOCK ); // while not close block
            }
            break;

         case ID_SETLIGHTLEVEL:
            {
               if ( ! mTryReadToken ( oszStr, ID_SETLIGHTLEVEL, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, TRUE, TRUE ) )
                  return 0;

               // define level as a number
               float
                  fLevel = (float) atof ( (char*) oszStr );
               
               m_poScen->mSetLightLevel ( fLevel );
            }
            break;

         case ID_SETLIGHTTABLE:
            {
               if ( ! mTryReadToken ( oszStr, ID_SETLIGHTTABLE, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, TRUE, TRUE ) )
                  return 0;

               // define level as a number
               DWORD
                  dwID = (DWORD) atoi ( (char*) oszStr );
               
               m_poScen->mSetLightTable ( dwID );
            }
            break;
                  
         case ID_LIGHTTABLES:
            {
               // try read open block, if can't return
               if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

                  #if COMP_SHOWLOADING >= 1
                  printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_LIGHTTABLES );
                  #endif

                  return 0;
               }

               // load light table files (one at a time)
               do {
                  // try read light table identifier, if error: exit
                  if ( ! mTryReadToken ( oszStr, ID_LIGHTTABLES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, FALSE, FALSE ) )
                     return 0;

                  // verify if not read end of block
                  if ( oszStr != STR_CLOSEBLOCK ) {
                     // define ID as a number
                     DWORD
                        dwID = (DWORD) atoi ( (char*) oszStr );

                     // try read light table file name, if error exit
                     if ( ! mTryReadToken ( oszStr, ID_LIGHTTABLES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_FILE_NAME, oszFile, FALSE, FALSE ) )
                        return 0;

                     // convert 3DFC file separator '/' to system file seperator
                     oszStr.mReplace ( _3DFC_FILESEPARATOR, _SYSTEM_FILESEPARATOR );

                     // try define file name (load is made bellow)
                     // define file name to be readed
                     CString
                        oszPath;
                     // verify if the name readed not have drive-separator
                     if ( ! oszStr.mTestFromLeft ( DRIVESEPARATOR ) ) {
                        // define relative path
                        CString
                           oszNot;
                        oszPath = oszFile;
                        if ( ! oszPath.mSplitFromRight ( _SYSTEM_FILESEPARATOR, oszNot ) )
                           oszPath = "";   // no path
                        else { // add file separator at end of path
                           char
                              szFileSep [ 2 ] = { _SYSTEM_FILESEPARATOR, 0 };  // construct string
                           oszPath += szFileSep;
                        }
                     }
                     // define all file name (with path)
                     CString
                        oszRead;
                     oszRead = oszPath + oszStr;

                     #if COMP_SHOWLOADING >= 2
                     printf ( MSG_READING_LIGHTTABLE, (char*) oszRead );
                     #endif

		     // try load light table from file (if can't is
                     // regenerated by CScenery)
                     CLightTable
                        oLgT;
                     oLgT.mCreateLoading ( oszRead );

                     // Success or fail nevermind, the next data are read
		     // everytime:

		     // try read the number of light levels
                     if ( ! mTryReadToken ( oszStr, ID_LIGHTTABLES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
                        return 0;

                     // define light levels as a number
                     DWORD
                        dwNumLevels = (DWORD) atoi ( (char*) oszStr );

                     // try read the light source Red color component
                     if ( ! mTryReadToken ( oszStr, ID_LIGHTTABLES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
                        return 0;

                     // define as a floating number
                     float
                        fNormR = (float) atof ( (char*) oszStr );

                     // try read the light source Green color component
                     if ( ! mTryReadToken ( oszStr, ID_LIGHTTABLES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
                        return 0;

                     // define as a floating number
                     float
                        fNormG = (float) atof ( (char*) oszStr );

                     // try read the light source Blue color component
                     if ( ! mTryReadToken ( oszStr, ID_LIGHTTABLES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
                        return 0;

                     // define as a floating number
                     float
                        fNormB = (float) atof ( (char*) oszStr );

                     // try add in scenery
                     if ( ! m_poScen->mAddLightTable ( oLgT, dwID, oszRead, dwNumLevels,
                                                       fNormR, fNormG, fNormB ) )
                        return 0;
                  }
               } while ( oszStr != STR_CLOSEBLOCK ); // while not close block
            }

            break;

         case ID_TEXTURES:
            // try read open block, if can't return
            if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

               #if COMP_SHOWLOADING >= 1
               printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_TEXTURES );
               #endif

               return 0;
            }

            // load textures from PCX files (one at a time)
            do {

               // try read texture identifier, if error: exit
               if ( ! mTryReadToken ( oszStr, ID_TEXTURES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, FALSE, FALSE ) )
                  return 0;

               // verify if not read end of block
               if ( oszStr != STR_CLOSEBLOCK ) {

                  // define ID as a number
                  DWORD
                     dwID = (DWORD) atoi ( (char*) oszStr );

                  // try read texture file name, if error exit
                  if ( ! mTryReadToken ( oszStr, ID_TEXTURES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_FILE_NAME, oszFile, FALSE, FALSE ) )
                     return 0;

                  // convert 3DFC file separator '/' to system file seperator
                  oszStr.mReplace ( _3DFC_FILESEPARATOR, _SYSTEM_FILESEPARATOR );

                  // read texture file
                  // define file name to be readed
                  CString
                     oszPath;
                    // verify if the name readed not have drive-separator
                  if ( ! oszStr.mTestFromLeft ( DRIVESEPARATOR ) ) {
                     // define relative path
                     CString
                        oszNot;
                     oszPath = oszFile;
                     if ( ! oszPath.mSplitFromRight ( _SYSTEM_FILESEPARATOR, oszNot ) )
                        oszPath = "";   // no path
                     else { // add file separator at end of path
                        char
                           szFileSep [ 2 ] = { _SYSTEM_FILESEPARATOR, 0 };  // construct string
                        oszPath += szFileSep;
                     }
                  }
                  // define all file name (with path)
                  CString
                     oszRead;
                  oszRead = oszPath + oszStr;
                  // load PCX file
                  CFilePCX
                     oFile;
                  CTexture
                     oTx;

                  #if COMP_SHOWLOADING >= 2
                  printf ( MSG_READING_TEXTURE, (char*) oszRead );
                  #endif

                  if ( ! oFile.mLoadTexture ( oszRead, oTx ) ) { // try load

                     #if COMP_SHOWLOADING >= 1
                     printf ( MSG_ERROR_NOT_FOUND, (char*) oszRead );
                     #endif

                     return 0;
                  }

                  // try add in scenery
                  if ( ! m_poScen->mAddTexture ( dwID, oTx ) )
                     return 0;
               }
            } while ( oszStr != STR_CLOSEBLOCK ); // while not close block

            break;

         case ID_START:
            {
               // try read camera identifier, if error: exit
               if ( ! mTryReadToken ( oszStr, ID_START, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, FALSE, FALSE ) )
                  return 0;

               // define start instance ID to be used as viewer camera as a number
               DWORD
                  dwCameraID = (DWORD) atoi ( (char*) oszStr );

               // try read background color (used to clear frame buffer), if error: exit
               if ( ! mTryReadToken ( oszStr, ID_START, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
                  return 0;

               // define background clear color
               INT
                  nClear = (DWORD) atoi ( (char*) oszStr );

               // verify if an error found
               if ( ! m_poScen->mSetStart ( dwCameraID, nClear ) )
                  return 0;
            }

            break;

         case ID_OBJECTS:
            // try read open block, if can't return
            if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

               #if COMP_SHOWLOADING >= 1
               printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_OBJECTS );
               #endif

               return 0;
            }

            // load identifies that must be instanceated and descriptions
            do {

               // try read intance ID (name of the object), if error: exit
               if ( ! mTryReadToken ( oszStr, ID_OBJECTS, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
                  return 0;

               // verify if not read end of block
               if ( oszStr != STR_CLOSEBLOCK ) {

                  // define ID as a number
                  DWORD
                     dwInstID = (DWORD) atoi ( (char*) oszStr );

                  // try read where place the instance (inside the sector
                  // of what define data), if error exit. Here read the destin instance ID.
                  if ( ! mTryReadToken ( oszStr, ID_OBJECTS, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, TRUE ) )
                     return 0;

                  // convert to number
                  DWORD
                     dwDestinInstID = (DWORD) atoi ( (char*) oszStr );

                  // try read the destin sector ID
                  if ( ! mTryReadToken ( oszStr, ID_OBJECTS, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, TRUE ) )
                     return 0;

                  // convert to number
                  DWORD
                     dwDestinSectorID = (DWORD) atoi ( (char*) oszStr );

                  // list of defines that compose the object
                  CList
                     oDefs;
                  INT
                     nCode = mReadList ( oDefs, ID_OBJECTS, oszFile );

                  // if error
                  if ( nCode == 0 )
                     return nCode;

                  // try add in scenery
                  if ( ! m_poScen->mAddObject ( dwInstID, dwDestinInstID,
                                                dwDestinSectorID,
                                                oDefs ) )
                     return 0;
               }
            } while ( oszStr != STR_CLOSEBLOCK ); // while not close block

            break;

         case ID_DEFINE:
            {
               // try read define identifier, if error: exit
               if ( ! mTryReadToken ( oszStr, ID_DEFINE, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, FALSE, FALSE ) )
                  return 0;

               // define define ID as a number
               DWORD
                  dwDefineID = (DWORD) atoi ( (char*) oszStr );

               // try read open block, if can't return
               if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

                  #if COMP_SHOWLOADING >= 1
                  printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_DEFINE );
                  #endif

                  return 0;
               }

               // show define ID

               #if COMP_SHOWLOADING == 2
               printf ( MSG_DEFINE_ID, dwDefineID );
               #endif
               
               // try create define in scenery
               INT
                  nCreateCode = m_poScen->mBeginAddDefine ( dwDefineID );
               // verify creation error
               if ( nCreateCode != 1 )
                  return nCreateCode;

               // load define declarations (blocks inside define)
               INT
                  nReturnCode = mReadBlock ( oszFile, ID_DEFINE );
               if ( nReturnCode != 1 ) {

                  #if COMP_SHOWLOADING >= 1
                  printf ( MSG_ERROR_READING_BLOCK, STR_DEFINE, dwDefineID );
                  #endif

                  return nReturnCode;
               }

               // try signals end define read
               INT
                  nEndCode = m_poScen->mEndAddDefine ();
               // verify error
               if ( nEndCode != 1 )
                  return nEndCode;
            }

            break;

         case ID_CLOSEBLOCK:
            bEnd = TRUE;
            break;

         case ID_SECTOR:
            {
               // try read sector identifier, if error: exit
               if ( ! mTryReadToken ( oszStr, ID_SECTOR, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_ID, oszFile, FALSE, FALSE ) )
                  return 0;

               // define sector ID as a number
               DWORD
                  dwSectorID = (DWORD) atoi ( (char*) oszStr );

               // try read open block, if can't return
               if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

                  #if COMP_SHOWLOADING >= 1
                  printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_SECTOR );
                  #endif

                  return 0;
               }

               // try create sector in current define data
               INT
                  nCreateCode = m_poScen->mBeginAddSector ( dwSectorID );
               // verify creation error
               if ( nCreateCode != 1 )
                  return nCreateCode;

               // show sector ID
               #if COMP_SHOWLOADING == 2
               printf ( MSG_SECTOR_ID, dwSectorID );
               #endif

               // load sector declarations (blocks inside sector)
               INT
                  nReturnCode = mReadBlock ( oszFile, ID_SECTOR );
               if ( nReturnCode != 1 ) {

                  #if COMP_SHOWLOADING >= 1
                  printf ( MSG_ERROR_READING_BLOCK, STR_SECTOR, dwSectorID );
                  #endif

                  return nReturnCode;
               }

               // try signals end sector read
               INT
                  nEndCode = m_poScen->mEndAddSector ();
               // verify error
               if ( nEndCode != 1 )
                  return nEndCode;
            }

            break;

         case ID_VERTEXES:
            // try read open block, if can't return
            if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

               #if COMP_SHOWLOADING >= 1
               printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, STR_VERTEXES );
               #endif

               return 0;
            }

            // load vertexes from file
            do {

               // try read X, if error exit
               if ( ! mTryReadToken ( oszStr, ID_VERTEXES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
                  return 0;

               // verify if not read end of block
               if ( oszStr != STR_CLOSEBLOCK ) {

                  // define X as a number
                  float
                     fX = (float) atof ( (char*) oszStr );

                  // try read Y, if error exit
                  if ( ! mTryReadToken ( oszStr, ID_VERTEXES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, TRUE, FALSE ) )
                     return 0;

                  // convert to number
                  float
                     fY = (float) atof ( (char*) oszStr );

                  // try read Z, if error exit
                  if ( ! mTryReadToken ( oszStr, ID_VERTEXES, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, TRUE, FALSE ) )
                     return 0;

                  // convert to number
                  float
                     fZ = (float) atof ( (char*) oszStr );
                  // try add in scenery
                  if ( ! m_poScen->mAddVertex ( fX, fY, fZ ) )
                     return 0;
               }
            } while ( oszStr != STR_CLOSEBLOCK ); // while not close block

            break;

         case ID_POLYGONS:
            {
               // try read open block, if can't return
               if ( ! mTestNextToken ( STR_OPENBLOCK ) )
                  return 0;

               // list of vertexes for one polygon
               CList
                  oIndexes;
               INT
                  nCode;                  

               // load polygons from file
               while ( ( nCode = mReadList ( oIndexes, ID_POLYGONS, oszFile ) ) == 1 )

                  // try add in scenery
                  if ( ! m_poScen->mAddPolygon ( oIndexes ) )
                     return 0;

               if ( nCode == 0 )
                  return nCode;

               break;
            }

         case ID_CMD_CREATE_COMPONENT:
            {
               // read slot ID, type ID, parameter 1 and 2
               DWORD
                  adwVals [ 4 ];
               if ( ! mReadArray ( 4, adwVals, ID_CMD_CREATE_COMPONENT, oszFile ) )
                  return 0;

               // read the list with component data
               CList
                  oCompParams;
               INT
                  nCode = mReadList ( oCompParams, ID_CMD_CREATE_COMPONENT, oszFile );

               if ( nCode == 0 )
                  return nCode;

               // try add in scenery
               m_poScen->mCreateComponent ( adwVals [ 0 ], adwVals [ 1 ], adwVals [ 2 ], adwVals [ 3 ], (PTRVAL) &oCompParams );

               break;
            }

         case ID_CMD_ACTIVATE_COMPONENT:
            {
               // read slot ID, parameter 1 and 2
               DWORD
                  adwVals [ 3 ];
               if ( ! mReadArray ( 3, adwVals, ID_CMD_CREATE_COMPONENT, oszFile ) )
                  return 0;

               // read the list with component data
               CList
                  oCompParams;
               INT
                  nCode = mReadList ( oCompParams, ID_CMD_ACTIVATE_COMPONENT, oszFile );

               if ( nCode == 0 )
                  return nCode;

               // try add in scenery
               m_poScen->mActivateComponent ( adwVals [ 0 ], adwVals [ 1 ], adwVals [ 2 ], (PTRVAL) &oCompParams );

               break;
            }

         case ID_CMD_DESTROY_COMPONENT:
            {
               // read slot ID, parameter 1 and 2
               DWORD
                  adwVals [ 3 ];
               if ( ! mReadArray ( 3, adwVals, ID_CMD_CREATE_COMPONENT, oszFile ) )
                  return 0;

               // read the list with component data
               CList
                  oCompParams;
               INT
                  nCode = mReadList ( oCompParams, ID_CMD_DESTROY_COMPONENT, oszFile );

               if ( nCode == 0 )
                  return nCode;

               // try add in scenery
               m_poScen->mDestroyComponent ( adwVals [ 0 ], adwVals [ 1 ], adwVals [ 2 ], (PTRVAL) &oCompParams );

               break;
            }

         case ID_SOLIDSTATE:
            {
               INT
                  nEndCode;
               if ( ( nEndCode = mReadValuesBlock ( oszFile, ID_SOLIDSTATE, 1 ) ) != 1 )
                  return nEndCode;
            }
            break;

         case ID_PORTALS:
            {
               INT
                  nEndCode;
               if ( ( nEndCode = mReadValuesBlock ( oszFile, ID_PORTALS, 2 ) ) != 1 )
                  return nEndCode;
            }
            break;

         case ID_POLYGONSLIGHT:
            {
               INT
                  nEndCode;
               if ( ( nEndCode = mReadValuesBlock ( oszFile, ID_POLYGONSLIGHT, 2 ) ) != 1 )
                  return nEndCode;
            }
            break;

         case ID_POLYGONSSURFACE:
            {
               INT
                  nEndCode;
               if ( ( nEndCode = mReadValuesBlock ( oszFile, ID_POLYGONSSURFACE, 1 ) ) != 1 )
                  return nEndCode;
            }
            break;
      }
   }

   return 1;
}



/* Read a single value block that follow the format:
{ val1 ... valn ndxpol1 [; | .. ndxpol2] ... }
Usefull to load portal settings, poly surface settings,
poly solid state,...
oszFile - file beeing load
nBlock - ID of block type to read
yNumVals - the number of values to read for the specific
 block type. SOLIDSTATE blocks have only one data value: the flag.
return 1 - ok / not 1 - error
**********************************************************/
INT CFileTXE::mReadValuesBlock ( CString oszFile, INT nBlock, BYTE yNumDataVals )
{
   // try read open block, if can't return
   if ( ! mTestNextToken ( STR_OPENBLOCK ) ) {

      #if COMP_SHOWLOADING >= 1
      printf ( MSG_ERROR_SPECTED, STR_OPENBLOCK, (char*) m_aoIDs [ nBlock ] );
      #endif

      return 0;
   }

   // store the data read
   float
      afData [ 256 ];
   // token string to be read
   CString
      oszStr;

   // read data while a close block not found
   do {

      // reset the data array
      for ( DWORD dwcD = 0; dwcD < 255; dwcD ++ )
         afData [ dwcD ] = 0;
      // read the specific number of values
      for ( BYTE ycV = 0; ycV < yNumDataVals; ycV ++ ) {

         // try read next
         if ( ! mTryReadToken ( oszStr, nBlock, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, TRUE, FALSE ) )
            return 0;
         // if not read the close block
         if ( oszStr != STR_CLOSEBLOCK )

            // convert value to float
            afData [ ycV ] = (float) atof ( (char*) oszStr );

         else
            break;
      }

      // if not read close block
      if ( oszStr != STR_CLOSEBLOCK ) {

         // try read first polygon index, if error exit
         if ( ! mTryReadToken ( oszStr, nBlock, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, TRUE, TRUE ) )
            return 0;

         // convert to number (index) - can have 2 indexes per value group
         DWORD
            dwNdxPol1 = (DWORD) atoi ( (char*) oszStr ),
            dwNdxPol2 = dwNdxPol1;

         // try read first polygon index, if error exit
         if ( ! mTryReadToken ( oszStr, nBlock, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED_IN, MSG_STR_VALUE, oszFile, FALSE, TRUE ) )
            return 0;

         // define what have been read
         // if not read an attribute separator, supose read trash
         if ( oszStr != STR_TUPLESEPARATOR ) {

            // verify if read continuator, if no -> abort
            if ( oszStr != STR_CONTINUATOR ) {

               #if COMP_SHOWLOADING >= 1
               printf ( MSG_ERROR_SPECTED_IN, STR_CONTINUATOR, (char*) m_aoIDs [ nBlock ] );
               #endif

               return 0;
            }

            // try read second polygon index, if error exit
            if ( ! mTryReadToken ( oszStr, nBlock, MSG_ERROR_SPECTED_IN, MSG_ERROR_SPECTED_IN, MSG_STR_VALUE, oszFile, TRUE, FALSE ) )
               return 0;

            // convert to number (index of an other polygon)
            dwNdxPol2 = (DWORD) atoi ( (char*) oszStr );
         }

         // try set data in scenery
         if ( ! m_poScen->mSetDataValues ( nBlock, (float*) afData, dwNdxPol1, dwNdxPol2 ) )
            return 0;
      }
   } while ( oszStr != STR_CLOSEBLOCK ); // while not close block

   return 1;
}



/* Try read a token. If the token is the same as
oszSpected return TRUE else FALSE.
**********************************************************/
BOOL CFileTXE::mTestNextToken ( CString oszSpected )
{
   // try read a token
   CString
      oszStr;
   if ( ! mRetStr ( oszStr ) )
      return FALSE;

   // verify if string is the spected string
   if ( oszStr == oszSpected )
      return TRUE;

   return FALSE;
}



/* Help make common tests when reading one string (token)
 from the file.
oszStr - variable where store the readed data
nContents - inside what block the read is (is the ID)
oszMsgError1 - the main message error if found an error when reading
oszMsgError2 - main message error when testing tuple separator
oszMsgWanted - complementar message error if found an error
oszFile - file beeing read
bTestTupleSeparator - test if found a tuple separator as in wrong place?
                      TRUE - yes / FALSE - no
bTestCloseBlock - test if found a close block as in wrong place?
                  TRUE - yes / FALSE - no
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CFileTXE::mTryReadToken ( CString& oszStr, INT nContents,
                               CString oszMsgError1,
                               CString oszMsgError2,
                               CString oszMsgWanted,
                               CString& oszFile,
                               BOOL bTestTupleSeparator,
                               BOOL bTestCloseBlock )
{
   // try read token
   if ( ! mRetStr ( oszStr ) ) {

      #if COMP_SHOWLOADING >= 1
      printf ( (char*) oszMsgError1, (char*) oszMsgWanted, (char*) m_aoIDs [ nContents ] );
      #endif

      return FALSE;
   }

   // test if read tuple separator
   if ( bTestTupleSeparator && oszStr == STR_TUPLESEPARATOR ) {

      #if COMP_SHOWLOADING >= 1
      printf ( (char*) oszMsgError2, (char*) oszMsgWanted, (char*) m_aoIDs [ nContents ] );
      #endif

      return FALSE;
   }

   // test if read close block
   if ( bTestCloseBlock && oszStr == STR_CLOSEBLOCK ) {

      #if COMP_SHOWLOADING >= 1
      printf ( MSG_ERROR_UNSPECTED_EOB, (char*) oszFile, (char*) m_aoIDs [ nContents ] );
      #endif

      return FALSE;
   }

   return TRUE;
}



/* Read a list of values. Stop when is found a tuple-separator (;).
oL - the list with readed data
nContents - inside what block the read is (is the ID)
oszFile - file beeing read
return 0 - error / 1 - ok / 2 - ok and close block found
       oL - the values (nodes type: CNodeFloat)
**********************************************************/
BYTE CFileTXE::mReadList ( CList& oL, INT nContents, CString& oszFile )
{
   CString
      oszStr;  // string read

   // empty list
   oL.mRemoveAll ();

   // try read first value
   if ( ! mTryReadToken ( oszStr, nContents, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, FALSE ) )
      return 0;

   // if the list has no elements
   if ( oszStr == STR_TUPLESEPARATOR )
      return 1;

   if ( oszStr == STR_CLOSEBLOCK )
      return 2;

   // execute while not found a tuple separator
   do {

      // convert to numeric value and insert in list
      CNodeFloat
         *poVal = new CNodeFloat;
      // verify if not alocate memory
      if ( poVal == NULL ) {

         #if COMP_SHOWLOADING >= 1
         printf ( MSG_ERROR_MEMORY_LOW );
         #endif

         return 0;
      }
      // convert
      poVal->m_fValue = (float) atof ( (char*) oszStr );
      // insert
      oL.mSetEnd ( poVal );

      // try read next token, if error exit
      if ( ! mTryReadToken ( oszStr, nContents, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, FALSE, TRUE ) )
         return 0;

   } while ( oszStr != STR_TUPLESEPARATOR ); // end of values

   return TRUE;
}



/* Read an array of values. Stop when the read all the values.
dwNumRead - number of elements to read
padwValues - pointer to the array of DWORDs to be filled
nContents - inside what block the array is (is the ID)
oszFile - file beeing read
return FALSE - error / TRUE - ok
       pafValues - the read values (nodes type: CNodeFloat)
**********************************************************/
BOOL CFileTXE::mReadArray ( DWORD dwNumRead, DWORD* padwValues, INT nContents, CString& oszFile )
{
   CString
      oszStr;  // string read

   // while not end read all values
   DWORD
      *pdwCursor = padwValues;   // array walker
   for ( DWORD dwcI = 0; dwcI < dwNumRead; dwcI ++ ) {

      // try read value
      if ( ! mTryReadToken ( oszStr, nContents, MSG_ERROR_SPECTED, MSG_ERROR_SPECTED, MSG_STR_VALUE, oszFile, TRUE, TRUE ) )
         return FALSE;

      // convert to numeric value and insert in array
      *pdwCursor = (DWORD) atoi ( (char*) oszStr );
      
      // if this is not the last element to read, goto next
      if ( dwcI < dwNumRead - 1 )
         pdwCursor ++;
   }

   return TRUE;
}



/* Define ID names in array (to user messages)
**********************************************************/
void CFileTXE::mDefineIDNames ()
{
   m_aoIDs [ ID_ERROR                  ] = STR_ERROR;
   m_aoIDs [ ID_NOBLOCK                ] = STR_NOBLOCK;
   m_aoIDs [ ID_DATA                   ] = STR_DATA;
   m_aoIDs [ ID_READ                   ] = STR_READ;
   m_aoIDs [ ID_TEXTURES               ] = STR_TEXTURES;
   m_aoIDs [ ID_PALETTE                ] = STR_PALETTE;
   m_aoIDs [ ID_DEFINE                 ] = STR_DEFINE;
   m_aoIDs [ ID_SECTOR                 ] = STR_SECTOR;
   m_aoIDs [ ID_POLYGONS               ] = STR_POLYGONS;
   m_aoIDs [ ID_VERTEXES               ] = STR_VERTEXES;
   m_aoIDs [ ID_PORTALS                ] = STR_PORTALS;
   m_aoIDs [ ID_SOLIDSTATE             ] = STR_SOLIDSTATE;
   m_aoIDs [ ID_OPENBLOCK              ] = STR_OPENBLOCK;
   m_aoIDs [ ID_CLOSEBLOCK             ] = STR_CLOSEBLOCK;
   m_aoIDs [ ID_OPENTUPLE              ] = STR_OPENTUPLE;
   m_aoIDs [ ID_CLOSETUPLE             ] = STR_CLOSETUPLE;
   m_aoIDs [ ID_OBJECTS                ] = STR_OBJECTS;
   m_aoIDs [ ID_START                  ] = STR_START;
   m_aoIDs [ ID_LIGHTTABLES            ] = STR_LIGHTTABLES;
   m_aoIDs [ ID_POLYGONSLIGHT          ] = STR_POLYGONSLIGHT;
   m_aoIDs [ ID_POLYGONSSURFACE        ] = STR_POLYGONSSURFACE;
   m_aoIDs [ ID_CMD_CREATE_COMPONENT   ] = STR_CMD_CREATE_COMPONENT;
   m_aoIDs [ ID_CMD_ACTIVATE_COMPONENT ] = STR_CMD_ACTIVATE_COMPONENT;
   m_aoIDs [ ID_CMD_DESTROY_COMPONENT  ] = STR_CMD_DESTROY_COMPONENT;
   m_aoIDs [ ID_INITMESSAGES           ] = STR_INITMESSAGES;
   m_aoIDs [ ID_SETLIGHTLEVEL          ] = STR_SETLIGHTLEVEL;
   m_aoIDs [ ID_SETLIGHTTABLE          ] = STR_SETLIGHTTABLE;
}



/* Define the type of block that the token correspond.
oszStr - the token to be tested
nParentBlock - context, the parent block
               (block where this block is inside)
The ID values can be found in file src/defs.h.
Acording the context one block can't be placed (it is an error).
Close block is one possible block type.
return ID type for the block or ID_ERROR if block in wrong place.
**********************************************************/
INT CFileTXE::mClassifyBlock ( CString oszStr, INT nParentBlock )
{
   // verify cases

   if ( oszStr == STR_READ )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_READ;
      else
         return ID_ERROR;

   if ( oszStr == STR_OBJECTS )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_OBJECTS;
      else
         return ID_ERROR;

   if ( oszStr == STR_START )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_START;
      else
         return ID_ERROR;

   if ( oszStr == STR_TEXTURES )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_TEXTURES;
      else
         return ID_ERROR;

   if ( oszStr == STR_PALETTE )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_PALETTE;
      else
         return ID_ERROR;

   if ( oszStr == STR_LIGHTTABLES )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_LIGHTTABLES;
      else
         return ID_ERROR;

   if ( oszStr == STR_INITMESSAGES )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_INITMESSAGES;
      else
         return ID_ERROR;

   if ( oszStr == STR_DEFINE )
      // verify context
      if ( nParentBlock == ID_NOBLOCK )
         return ID_DEFINE;
      else
         return ID_ERROR;

   if ( oszStr == STR_SECTOR )
      // verify context
      if ( nParentBlock == ID_DEFINE )
         return ID_SECTOR;
      else
         return ID_ERROR;

   // close block is one possible name!
   if ( oszStr == STR_CLOSEBLOCK )
      // verify context
      if ( nParentBlock == ID_DEFINE || nParentBlock == ID_SECTOR )
         return ID_CLOSEBLOCK;
      else
         return ID_ERROR;

   if ( oszStr == STR_POLYGONS )
      // verify context
      if ( nParentBlock == ID_DEFINE || nParentBlock == ID_SECTOR )
         return ID_POLYGONS;
      else
         return ID_ERROR;

   if ( oszStr == STR_VERTEXES )
      // verify context
      if ( nParentBlock == ID_DEFINE || nParentBlock == ID_SECTOR )
         return ID_VERTEXES;
      else
         return ID_ERROR;

   if ( oszStr == STR_PORTALS )
      // verify context
      if ( nParentBlock == ID_DEFINE || nParentBlock == ID_SECTOR )
         return ID_PORTALS;
      else
         return ID_ERROR;

   if ( oszStr == STR_SOLIDSTATE )
      // verify context
      if ( nParentBlock == ID_DEFINE || nParentBlock == ID_SECTOR )
         return ID_SOLIDSTATE;
      else
         return ID_ERROR;

   if ( oszStr == STR_POLYGONSLIGHT )
      // verify context
      if ( nParentBlock == ID_DEFINE || nParentBlock == ID_SECTOR )
         return ID_POLYGONSLIGHT;
      else
         return ID_ERROR;

   if ( oszStr == STR_POLYGONSSURFACE )
      // verify context
      if ( nParentBlock == ID_DEFINE || nParentBlock == ID_SECTOR )
         return ID_POLYGONSSURFACE;
      else
         return ID_ERROR;

   if ( oszStr == STR_CMD_CREATE_COMPONENT )
      // verify context
      if ( nParentBlock == ID_DEFINE )
         return ID_CMD_CREATE_COMPONENT;
      else
         return ID_ERROR;

   if ( oszStr == STR_CMD_ACTIVATE_COMPONENT )
      // verify context
      if ( nParentBlock == ID_DEFINE )
         return ID_CMD_ACTIVATE_COMPONENT;
      else
         return ID_ERROR;

   if ( oszStr == STR_CMD_DESTROY_COMPONENT )
      // verify context
      if ( nParentBlock == ID_DEFINE )
         return ID_CMD_DESTROY_COMPONENT;
      else
         return ID_ERROR;

   if ( oszStr == STR_SETLIGHTLEVEL )
      // verify context
      if ( nParentBlock == ID_DEFINE ||
           nParentBlock == ID_SECTOR ||
           nParentBlock == ID_NOBLOCK )
         return ID_SETLIGHTLEVEL;
      else
         return ID_ERROR;

   if ( oszStr == STR_SETLIGHTTABLE )
      // verify context
      if ( nParentBlock == ID_DEFINE ||
           nParentBlock == ID_SECTOR ||
           nParentBlock == ID_NOBLOCK )
         return ID_SETLIGHTTABLE;
      else
         return ID_ERROR;

   return ID_ERROR;
}
