# Use for LINUX - X11 target.
# Compile library source code, examples and utils.
# Create 3DFC object files and move them to LIB if use label: library.
# Create examples and move to EX directory if use label: examples.
# If you have problems using make.exe change the label names bellow.
# Compiler options:
#   -g     compile with debug
#   -O2    optimization 2
#   -m486  use 486 instructions or -mpentium use Pentium instructions
#   -Wall  show all warnings



COMPILER = gcc

OPTIONS  = -O3 -m486 -Wall -DTRG_LINUXX11 -ffast-math

EXTRA_LIBS = -lm -lXaw -lXmu -lXext -lXt -lX11 -ldl -L /usr/X11/lib

CMD_COPY_OBJ = cp *.o lib

CMD_DEL_OBJ  = rm *.o

CMD_COPY_EXE = cp *.exe ex

CMD_DEL_EXE  = rm *.exe

CMD_COPY_UTIL = cp *.exe util/bin

SRC_DIR  = src/
# components source
COM_DIR  = $(SRC_DIR)comp/

LIB_DIR = lib/

EX_DIR  = ex/

OBJ =			$(LIB_DIR)list.o     \
				$(LIB_DIR)heap.o     \
				$(LIB_DIR)queue.o    \
				$(LIB_DIR)map.o      \
				$(LIB_DIR)mem.o      \
				$(LIB_DIR)string.o   \
				$(LIB_DIR)file.o     \
				$(LIB_DIR)filepcx.o  \
				$(LIB_DIR)filerem.o  \
				$(LIB_DIR)filetxe.o  \
				$(LIB_DIR)texture.o  \
				$(LIB_DIR)screen.o   \
				$(LIB_DIR)draw.o     \
				$(LIB_DIR)lighttab.o \
				$(LIB_DIR)mesh.o     \
				$(LIB_DIR)sector.o   \
				$(LIB_DIR)object.o   \
				$(LIB_DIR)scenery.o  \
 				$(LIB_DIR)render.o   \
				$(LIB_DIR)camsys.o   \
 				$(LIB_DIR)engine.o   \
				$(LIB_DIR)bpn.o      \
				$(LIB_DIR)socket.o   \
				$(LIB_DIR)svsocket.o \
 				$(LIB_DIR)math.o     \
 				$(LIB_DIR)polygon.o	\
 				$(LIB_DIR)refsys.o	\
 				$(LIB_DIR)xwinapp.o	\
 				$(LIB_DIR)eventtab.o	\
 				$(LIB_DIR)cokbcont.o	\
 				$(LIB_DIR)codata.o	\
 				$(LIB_DIR)cocamsys.o	\
 				$(LIB_DIR)comesh.o	\
 				$(LIB_DIR)cosecset.o	\
            $(LIB_DIR)colsomni.o   \
				$(LIB_DIR)neuron.o   

HEADERS =	$(SRC_DIR)list.h     \
				$(SRC_DIR)heap.h     \
				$(SRC_DIR)queue.h    \
				$(SRC_DIR)map.h      \
				$(SRC_DIR)mem.h      \
				$(SRC_DIR)array.h    \
				$(SRC_DIR)string.h   \
				$(SRC_DIR)file.h     \
				$(SRC_DIR)filepcx.h  \
				$(SRC_DIR)filerem.h  \
				$(SRC_DIR)filetxe.h  \
				$(SRC_DIR)texture.h  \
				$(SRC_DIR)screen.h   \
				$(SRC_DIR)draw.h     \
				$(SRC_DIR)lighttab.h \
				$(SRC_DIR)mesh.h     \
				$(SRC_DIR)sector.h   \
				$(SRC_DIR)object.h   \
				$(SRC_DIR)scenery.h  \
				$(SRC_DIR)render.h   \
				$(SRC_DIR)camsys.h   \
 				$(SRC_DIR)engine.h   \
				$(SRC_DIR)bpn.h      \
				$(SRC_DIR)socket.h   \
				$(SRC_DIR)svsocket.h \
 				$(SRC_DIR)math.h	   \
 				$(SRC_DIR)refsys.h	\
 				$(SRC_DIR)polygon.h	\
            $(SRC_DIR)xwinapp.h  \
            $(SRC_DIR)eventtab.h \
 				$(COM_DIR)cokbcont.h	\
 				$(COM_DIR)codata.h	\
 				$(COM_DIR)colsomni.h	\
 				$(COM_DIR)cocamsys.h	\
 				$(COM_DIR)comesh.h	\
 				$(COM_DIR)cosecset.h	\
				$(SRC_DIR)neuron.h   



# show init info
info:
				@echo 3D Foundation Classes compiling instructions:
				@echo To compile all - library, examples and utils - use: make all
				@echo To compile only the library object files, use: make library
				@echo To compile only the examples, use: make examples
				@echo To compile only the utils, use: make utils
				@echo To compile a separate object file, example or util
				@echo pass its label in command line.
				@echo Ex.: make mem		  - compile the CMem library class
				@echo See readme.txt to get more info about how to configure 
				@echo compilation for distinct platforms like DOS or Linux.



# build all code available
all: 		   library examples utils
				@echo .
				@echo .
				@echo Library object files compiled and moved to LIB.
				@echo Examples compiled and moved to EX, for more info see docs/examples.txt
				@echo To compile only the library object files use: make library
				@echo To compile only the examples use: make examples
				@echo To compile a separate object file or example pass its label in command line.
				@echo Ex.: make mem		  - compile the CMem library class
				@echo 3D Foundation Classes compiled for Linux. All ready to use!!



# build only the library object files
library:	   list mem string heap queue file filerem \
            filepcx screen draw filetxe map scenery texture object \
            sector mesh camsys bpn neuron engine render socket svsocket \
				lighttab math polygon refsys cokbcont codata cocamsys \
				comesh cosecset colsomni xwinapp eventtab
				@echo .
				@echo .
				@echo Object files moved to LIB.
				@echo To compile a separate object file pass its label in command line.
				@echo Ex.: make mem		  - compile the CMem library class



# build only the library examples
examples:	exlist exmem exarray exstring \
				exheap exqueue exfile exfilerem \
				exshowpcx exscale exdraw extesttxe exmap \
			   exbpnand exbpnor exbpnxor exbpn exdrawclip exfly \
				exlight exdrawtex exdrawgou exdrawgt exxwinapp
				@echo .
				@echo .
				@echo Example files moved to EX.
				@echo To compile a separate example pass its label in command line.
				@echo Ex.: make exstring		  - compile the string example



# build only the utilitys
utils:		utbb
				@echo .
				@echo .
				@echo Util files moved to UTILS/BIN.
				@echo To compile a separate util pass its label in command line.
				@echo Ex.: make utmkins		  - compile the Make Install



# THE LIBRARY SOURCES
###########################################################



mem:			$(SRC_DIR)mem.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)mem.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

eventtab:	$(SRC_DIR)eventtab.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)eventtab.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

math:			$(SRC_DIR)math.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)math.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

colsomni:	$(COM_DIR)colsomni.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)colsomni.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

cokbcont:	$(COM_DIR)cokbcont.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)cokbcont.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

codata:		$(COM_DIR)codata.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)codata.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

comesh:		$(COM_DIR)comesh.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)comesh.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

cocamsys:	$(COM_DIR)cocamsys.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)cocamsys.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

cosecset:	$(COM_DIR)cosecset.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)cosecset.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

gr:			$(SRC_DIR)/linx11/gr.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)/linx11/gr.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

polygon:		$(SRC_DIR)polygon.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)polygon.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

refsys:		$(SRC_DIR)refsys.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)refsys.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

object:		$(SRC_DIR)object.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)object.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

sector:		$(SRC_DIR)sector.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)sector.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

mesh:	 		$(SRC_DIR)mesh.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)mesh.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

camsys:	 	$(SRC_DIR)camsys.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)camsys.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

engine:		 	$(SRC_DIR)engine.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)engine.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

render:	 	$(SRC_DIR)render.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)render.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

draw:			$(SRC_DIR)draw.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

lighttab:	$(SRC_DIR)lighttab.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)lighttab.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

screen:		$(SRC_DIR)screen.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)screen.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

texture:		$(SRC_DIR)texture.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)texture.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

filepcx:		$(SRC_DIR)filepcx.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)filepcx.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

filerem:		$(SRC_DIR)filerem.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)filerem.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

filetxe:		$(SRC_DIR)filetxe.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)filetxe.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

scenery:		$(SRC_DIR)scenery.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)scenery.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

file:			$(SRC_DIR)file.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)file.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

queue:		$(SRC_DIR)queue.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)queue.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

heap:			$(SRC_DIR)heap.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)heap.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

string:		$(SRC_DIR)string.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)string.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

list:     	$(SRC_DIR)list.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)list.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

map:     	$(SRC_DIR)map.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)map.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

neuron:    	$(SRC_DIR)neuron.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)neuron.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

bpn: 	   	$(SRC_DIR)bpn.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)bpn.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

socket:   	$(SRC_DIR)socket.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)socket.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)

svsocket:  	$(SRC_DIR)svsocket.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)svsocket.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



# THE EXAMPLES
###########################################################



exarray:		$(EX_DIR)array.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)array.cc
				$(COMPILER) $(OPTIONS) $(OBJ) array.o -o array.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exmem:		$(EX_DIR)mem.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)mem.cc
				$(COMPILER) $(OPTIONS) $(OBJ) mem.o -o mem.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exxwinapp:	$(EX_DIR)exxwinapp.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)exxwinapp.cc
				$(COMPILER) $(OPTIONS) $(OBJ) exxwinapp.o -o exxwinapp.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)
						  
exdrawgt:	$(EX_DIR)drawgt.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawgt.cc
				$(COMPILER) $(OPTIONS) $(OBJ) drawgt.o -o drawgt.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)
						  
exdrawgou:	$(EX_DIR)drawgou.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawgou.cc
				$(COMPILER) $(OPTIONS) $(OBJ) drawgou.o -o drawgou.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)
						  
exdrawtex:	$(EX_DIR)drawtex.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawtex.cc
				$(COMPILER) $(OPTIONS) $(OBJ) drawtex.o -o drawtex.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exlight:		$(EX_DIR)light.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)light.cc
				$(COMPILER) $(OPTIONS) $(OBJ) light.o -o light.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exlist:		$(EX_DIR)list.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)list.cc
				$(COMPILER) $(OPTIONS) $(OBJ) list.o -o list.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exstring:	$(EX_DIR)string.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)string.cc
				$(COMPILER) $(OPTIONS) $(OBJ) string.o -o string.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exheap:		$(EX_DIR)heap.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)heap.cc
				$(COMPILER) $(OPTIONS) $(OBJ) heap.o -o heap.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exqueue:		$(EX_DIR)queue.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)queue.cc
				$(COMPILER) $(OPTIONS) $(OBJ) queue.o -o queue.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exfile:		$(EX_DIR)file.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)file.cc
				$(COMPILER) $(OPTIONS) $(OBJ) file.o -o file.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exfilerem:	$(EX_DIR)filerem.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)filerem.cc
				$(COMPILER) $(OPTIONS) $(OBJ) filerem.o -o filerem.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exmap:		$(EX_DIR)map.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)map.cc
				$(COMPILER) $(OPTIONS) $(OBJ) map.o -o map.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

extesttxe:	$(EX_DIR)testtxe.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)testtxe.cc
				$(COMPILER) $(OPTIONS) $(OBJ) testtxe.o -o testtxe.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exdraw:		$(EX_DIR)draw.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)draw.cc
				$(COMPILER) $(OPTIONS) $(OBJ) draw.o -o draw.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exscale:		$(EX_DIR)scale.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)scale.cc
				$(COMPILER) $(OPTIONS) $(OBJ) scale.o -o scale.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exshowpcx:	$(EX_DIR)showpcx.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)showpcx.cc
				$(COMPILER) $(OPTIONS) $(OBJ) showpcx.o -o showpcx.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exfly:		$(EX_DIR)fly.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)fly.cc
				$(COMPILER) $(OPTIONS) $(OBJ) fly.o -o fly.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exdrawclip:	$(EX_DIR)drawclip.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawclip.cc
				$(COMPILER) $(OPTIONS) $(OBJ) drawclip.o -o drawclip.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exbpn:		$(EX_DIR)bpn.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpn.cc
				$(COMPILER) $(OPTIONS) $(OBJ) bpn.o -o bpn.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exbpnxor:	$(EX_DIR)bpnxor.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpnxor.cc
				$(COMPILER) $(OPTIONS) $(OBJ) bpnxor.o -o bpnxor.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exbpnor:		$(EX_DIR)bpnor.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpnor.cc
				$(COMPILER) $(OPTIONS) $(OBJ) bpnor.o -o bpnor.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exbpnand:	$(EX_DIR)bpnand.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpnand.cc
				$(COMPILER) $(OPTIONS) $(OBJ) bpnand.o -o bpnand.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exserver:	$(EX_DIR)server.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)server.cc
				$(COMPILER) $(OPTIONS) $(OBJ) server.o -o server.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

exclient:	$(EX_DIR)client.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)client.cc
				$(COMPILER) $(OPTIONS) $(OBJ) client.o -o client.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



# THE UTILS
###########################################################



utbb:			$(UTIL_DIR)bb/bb.cc $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(UTIL_DIR)bb/bb.cc
				$(COMPILER) $(OPTIONS) $(OBJ) bb.o -o bb.exe
				$(CMD_COPY_UTIL)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)
