/* X Window graphic/aplication management for XFree86.

Is possible:
create one app window, init app, set palette, close window,
update the window, set the event loop

TIPS:
- the reference system is the same as the video:   (0,0)
                                                      *----->Y
                                                      |
                                                      |
                                                      |
                                                      VX
- to create one XWindow app follow the steps: create the app with mCreate, set the
  event loop with mMakeLoop, end the app with mDestroy.
*/



#include <time.h> 
#include <sys/timeb.h>

#define XK_MISCELLANY 1
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysymdef.h>

#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>

#ifdef DO_SHM
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif /* DO_SHM */

#include "../defs.h"

#include "../mem.h"

#include "xwinapp.h"


// data to the X system
static XrmOptionDescRec astTABLE [] = {
   { "-flag",   "Flag",                XrmoptionNoArg,  (caddr_t) "on" },
   { "-param",  "parameter",           XrmoptionSepArg, NULL },
};

typedef struct {
   INT nFlag;
   INT nParameter;
   char *pyGeometry;
   INT nBorderwidth;
} pAppResourceRec, *pAppRes;

static pAppResourceRec stAPP_RESOURCES;
static char szWIN_DIMENSION [ 100 ];

static XtResource astRESOURCES [] = {
  { "param", "Parameter", XtRInt, sizeof ( INT ),
    XtOffset ( pAppRes, nParameter ), XtRImmediate, (caddr_t) 80},
  { "flag", "Flag", XtRBoolean, sizeof ( Boolean ),
    XtOffset ( pAppRes, nFlag ), XtRImmediate, (caddr_t) TRUE },
  { XtNgeometry,  "Geometry", XtRString, sizeof ( caddr_t ),
    XtOffset ( pAppRes, pyGeometry ), XtRString, (caddr_t) szWIN_DIMENSION },
  { XtNborderWidth, "borderWidth", XtRInt, sizeof ( INT ),
    XtOffset ( pAppRes, nBorderwidth ), XtRImmediate, (caddr_t) 1},
};



// implemented above
void EventsHandler (  Widget stWi, XtPointer stClosure, XEvent* pstEvent,
                      Boolean* bCont_to_dispatch );



/* constructor
**********************************************************/
CXWindowApp::CXWindowApp ()
{
   m_pstDisplay = NULL;
   m_stTopLevel = 0;
   m_stShell = 0;
   m_stBox = 0;

   // window data
   m_nScreenNum = 0;
   m_nDpyWidth = 0;
   m_nDpyHeight = 0;
   m_pXImg = NULL;
   m_pVid = NULL;

   #ifdef DO_SHM
   m_XShmSegmentInfo stShmi;
   m_stShmImg;
   #endif

   m_stColorMap = 0;
   szWIN_DIMENSION [ 0 ] = 0;
}



/* destructor
**********************************************************/
CXWindowApp::~CXWindowApp ()
{
}



/* Create a window, init event handler,...
**********************************************************/
void CXWindowApp::mCreate ( INT nArgc, char* pszArgv [] )
{
   Screen*
      pScreenPtr;
   Widget
      stBox;

   // define the resolution
   sprintf ( szWIN_DIMENSION, "%dx%d", RESX, RESY );

   // init the app and window
   
   m_stTopLevel = XtVaAppInitialize ( &m_stApContext, " ", astTABLE,
                  XtNumber ( astTABLE ), &nArgc, pszArgv, NULL, NULL, 0 );

   XtGetApplicationResources ( m_stTopLevel, &stAPP_RESOURCES, astRESOURCES,
                               XtNumber ( astRESOURCES ), NULL, (Cardinal) 0 );

   //XParseGeometry (stAPP_RESOURCES.geometry, &geom_x, &geom_y, &geom_width,
   //&geom_height);

   m_stShell = XtVaCreatePopupShell ( " ",
               topLevelShellWidgetClass,
               m_stTopLevel,
               XtNgeometry, stAPP_RESOURCES.pyGeometry,
               XtNborderWidth, stAPP_RESOURCES.nBorderwidth,
               0 );

   //stBox = XtCreateManagedWidget ("box", boxWidgetClass, stShell, NULL, (Cardinal) 0);
   m_stBox = XtCreateWidget ( "box", boxWidgetClass, m_stShell, NULL, (Cardinal) 0 );

   XtPopup ( m_stShell, XtGrabNone );
 
   m_pstDisplay = XtDisplay ( m_stShell );
   m_nScreenNum = DefaultScreen ( m_pstDisplay );
   pScreenPtr   = DefaultScreenOfDisplay ( m_pstDisplay );
   m_nDpyWidth  = DisplayWidth ( m_pstDisplay, m_nScreenNum );
   m_nDpyHeight = DisplayHeight ( m_pstDisplay, m_nScreenNum );
   m_stWindow   = XtWindow ( m_stShell );

   // configure drawing attributes
   XGCValues
      stValues;
   m_stGC = XCreateGC ( m_pstDisplay, m_stWindow, 0, &stValues );
   XSetForeground ( m_pstDisplay, m_stGC, BlackPixel ( m_pstDisplay, m_nScreenNum ) );
   XSetLineAttributes ( m_pstDisplay, m_stGC, 0, LineSolid, CapButt, JoinMiter );
   XSetGraphicsExposures ( m_pstDisplay, m_stGC, False );

   // create palette
   m_stColorMap = XCreateColormap ( m_pstDisplay, RootWindow ( m_pstDisplay, m_nScreenNum ), DefaultVisual ( m_pstDisplay, m_nScreenNum ), AllocAll );

   XtAddEventHandler ( m_stShell,
                       ExposureMask | KeyPressMask | KeyReleaseMask |
                       KeymapStateMask | FocusChangeMask |
                       PointerMotionMask | ButtonPressMask | ButtonReleaseMask, True,
                       EventsHandler, NULL );

   XMapWindow ( m_pstDisplay, XtWindow ( m_stShell ) );
   XSetWindowColormap ( m_pstDisplay, XtWindow ( m_stShell ), m_stColorMap );

   m_pXImg = NULL;

   #ifdef DO_SHM
   printf ( "SHM on\n" );
   #endif
   #ifndef DO_SHM
   printf ( "SHM off\n" );
   #endif
}



/* Close the window, quit the app
**********************************************************/
void CXWindowApp::mDestroy ()
{
  if ( m_stShell )
    XtPopdown ( m_stShell );
  if ( m_stTopLevel )
    XtDestroyWidget ( m_stTopLevel );
}



/* Init the app loop
**********************************************************/
void CXWindowApp::mMakeLoop ()
{
   //XtAppAddTimeOut ( m_stApContext, 300, AnimHandler, NULL );
   XtAppMainLoop ( m_stApContext );
}



/* Write a memory array of RGB data in palette.
**********************************************************/
void CXWindowApp::mWritePal ( CMem& oMem )
{
   XColor
      stColor;
   // seek all palette writing RGB
   for ( DWORD dwcCol = 0; dwcCol < 768; dwcCol += 3 ) {
      
      stColor.pixel = dwcCol / 3;
      stColor.red   = 256 * oMem.m_payMem [ dwcCol + 0 ];
      stColor.green = 256 * oMem.m_payMem [ dwcCol + 1 ];
      stColor.blue  = 256 * oMem.m_payMem [ dwcCol + 2 ];
      stColor.flags = DoRed | DoGreen | DoBlue;
      XStoreColor ( m_pstDisplay, m_stColorMap, &stColor );
   }
}



/* Show a frame buffer on window area.
**********************************************************/
void CXWindowApp::mShow ( CMem& oMem )
{
   #ifdef DO_SHM
   XShmPutImage ( m_pDiplay, m_stWindow, m_stGC, &m_stShmImg, 0, 0, 0, 0, RESX, RESY, False );
   #elseif
   XPutImage ( m_pDisplay, m_stWindow, m_stGC, m_pXImg, 0, 0, 0, 0, RESX, RESY );
   #endif
}



// Declare a XWindow objetc instance
CXWindowApp
   oAPP;



/* This is the event handler
**********************************************************/
void EventsHandler (  Widget stWi, XtPointer stClosure, XEvent* pstEvent,
                      Boolean* bCont_to_dispatch )
{
   KeySym
      stKey;
   int
      nKey2;
   int
      nState;
  
   //(void)w; (void)closure; (void)cont_to_dispatch;

   switch ( pstEvent->type ) {
      case Expose:
         if ( ! oAPP.m_pXImg ) {
            oAPP.m_pXImg = XGetImage ( oAPP.m_pstDisplay, oAPP.m_stWindow, 0, 0, RESX, RESY, AllPlanes, ZPixmap);
         #ifdef DO_SHM
            {
               oAPP.m_stShmImg = *oAPP.m_pXImg;
               oAPP.m_stShmi.shmid = shmget ( IPC_PRIVATE, oAPP.m_pXImg->bytes_per_line * oAPP.m_pXImg->height, IPC_CREAT | 0777 );
               oAPP.m_stShmi.shmaddr = (char*) shmat ( oAPP.m_stShmi.shmid, 0, 0 );
               oAPP.m_stShmi.readOnly = FALSE;
               XShmAttach ( oAPP.m_pDisplay, &oAPP.m_stShmi );
               oAPP.m_stShmImg.data = oAPP.m_stShmi.shmaddr;
               oAPP.m_pVid = (unsigned char*) oAPP.m_stShmi.shmaddr;
               oAPP.m_stShmImg.obdata = (char*) &oAPP.m_stShmi;
            }
         #endif /* DO_SHM */
            {
               oAPP.m_pVid = (unsigned char*) ( oAPP.m_pXImg->data );
            }
         }
         break;

      case KeyPress:
         stKey = XLookupKeysym ( (XKeyEvent*) pstEvent, 0 );
         nState = ( (XKeyEvent*) pstEvent )->state;
         nKey2 = stKey;
         switch ( stKey ) {
            case XK_Escape:
               oAPP.mDestroy ();
               exit ( 0 );
               break;
            default:
               break;
         }
         break;

      default:
         break;
   }
}


main ( int argc, char* argv [] )
{
  oAPP.mCreate ( argc, argv );
  printf ( "Iniciou\n" );
  oAPP.mMakeLoop ();
  printf ( "Acabou\n" );
  oAPP.mDestroy ();
}
