/* Implements a dynamic alocated memory buffer of bytes

FEATURES:
- you can realocate the buffer mantaning all data (if no loose of
  information) using CMem::mRealocate. Data is truncated if realocated
  array is smaller than the original array.
- you can use (void*) and others cast operators to convert buffer data and
  access buffer data directly.
- CMem::mDestroy do pointer point to string '(null)'!!!

TIPS:
- the index of first byte is 0
- the index of last byte is CMem::m_dwSize - 1
*/



#include <string.h>

#include "defs.h"

#include "mem.h"



/* constructor - init empty buffer
**********************************************************/
CMem::CMem ()
{
   m_payMem = NULL;
   m_dwSize = 0;
}



/* constructor - alocate a specific size buffer
**********************************************************/
CMem::CMem ( DWORD dwSize )
{
   m_payMem = NULL;
   m_dwSize = 0;

   mCreate ( dwSize );
}



/* constructor - init buffer with a copy of other buffer
**********************************************************/
CMem::CMem ( CMem& oMem )
{
   m_payMem = NULL;
   m_dwSize = 0;

   mCreate ( oMem.m_dwSize );

   memcpy ( (void*) m_payMem, (void*) oMem.m_payMem, m_dwSize );
}



/* destructor - desalocate buffer, if alocated
**********************************************************/
CMem::~CMem ()
{
   mDestroy ();
}



/* Create buffer doing dinamic alocation. If not enough memory
abort program execution. (like CMem::constructor)
**********************************************************/
void CMem::mCreate ( DWORD dwSize )
{
   mDestroy ();

   // try alocation
   if ( ( m_payMem = new BYTE [ dwSize ] ) == NULL )
      ABORT ( MSG_ERROR_MEMORY_LOW, __FILE__, __LINE__ )

   m_dwSize = dwSize;
}



/* Desalocate buffer, if alocated
**********************************************************/
void CMem::mDestroy ()
{
   // verify if it's alocated
   if ( m_payMem != NULL ) {
      delete m_payMem;
      m_payMem = NULL;
      m_dwSize = 0;
   }
}



/* Realocate buffer, copy the original elements to the new buffer.
If new buffer is smaller than the original buffer they are truncated.
dwSize - the size for the new buffer.
return TRUE - ok / FALSE error
**********************************************************/
BOOL CMem::mRealocate ( DWORD dwSize )
{
   // Create new memory buffer
   // try alocation
   BYTE
      *payMem;
   if ( ( payMem = new BYTE [ dwSize ] ) == NULL ) {
      printf ( MSG_ERROR_MEMORY_LOW );
      printf ( MSG_DEFAULT_ABORT_INFO, __FILE__, __LINE__ );
      return FALSE;
   }

   // copy elements to new buffer
   // if new buffer is short than the actual: truncate
   if ( dwSize < m_dwSize )
      memcpy ( (void*) payMem, (void*) m_payMem, dwSize );
   else
      // if new buffer have igual size or is greater: copy all
	 memcpy ( (void*) payMem, (void*) m_payMem, m_dwSize );

   // desalocate current buffer
   if ( m_payMem != NULL )
      delete m_payMem;

   // change current buffer to new one
   m_payMem = payMem;
   m_dwSize = dwSize;

   return TRUE;
}



/* Init memory buffer with a copy of other object
(like CMem::constructor and CMem::operator =)
**********************************************************/
void CMem::mSetObj ( CMem& oMem )
{
   mCreate ( oMem.m_dwSize );

   memcpy ( (void*) m_payMem, (void*) oMem.m_payMem, m_dwSize );
}



/* like CMem::constructor and CMem::mSetObj
**********************************************************/
void CMem::operator = ( CMem& oMem )
{
   mSetObj ( oMem );
}



/* Return a byte value
**********************************************************/
BYTE CMem::operator [] ( DWORD dwPos )
{
   return * ( m_payMem + dwPos );
}



/* Cast object to byte array, return memory array
**********************************************************/
CMem::operator BYTE* ()
{
   return m_payMem;
}



/* Cast object to void array, return memory array
**********************************************************/
CMem::operator void* ()
{
   return (void*) m_payMem;
}
