/* Implement a neural network neuron. Have weights, entrys and compute
the net.

 neuron:
                     X0 = +1           X - input
         W0                            W - weight
 X0--------\                           net = X0*W0 + X1*W1 + ...
            \                          Y = tanh ( net )
         W1  \
 X1-----------+---+  net +---+   Y
              | S |------| f |---
    ....      +---+      +---+
             /
 Xn---------/
         Wn
*/



#include <string.h>

#include <math.h>

#include "defs.h"

#include "node.h"

#include "nodeflo.h"

#include "list.h"

#include "nodelist.h"

#include "mem.h"

#include "string.h"

#include "file.h"

#include "neuron.h"

#include "nodenet.h"



/* constructor - init empty
**********************************************************/
CNeuron::CNeuron ()
{
}



/* destructor
**********************************************************/
CNeuron::~CNeuron ()
{
   mDestroy ();
}



/* Init neuton linking it to a previous layer and alocate weights.
bInFirstLayer - TRUE if neuron is in first hiden layer
oPreviousInput - input for the neuron
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CNeuron::mCreate ( CNodeList& oPreviousInput )
{
   mDestroy ();

   m_oX.m_oList.mSetObj ( oPreviousInput.m_oList );

   return mCreateWeights ( m_oX.m_oList.mRetNumNodes () + 1 ); // +1 to include bias
}



/* Alocate and init weights.
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CNeuron::mCreateWeights ( DWORD dwNumWeights )
{
   // try alocate memory to weight list
   for ( DWORD dwcN = 0; dwcN < dwNumWeights; dwcN ++ ) {

      // try alocate memory
      CNodeFloat
         *poWeight = new CNodeFloat;
      if ( poWeight == NULL ) {
         printf ( MSG_ERROR_MEMORY_LOW );
         printf ( MSG_DEFAULT_ABORT_INFO, __FILE__, __LINE__ );
         return FALSE;
      }

      // set default weight value to a random number between 0 and 1
      poWeight->m_fValue = ( (float) ( random () % 1000 ) / 1000 ) - 0.5;
      // insert weight in list
      m_oW.mSetEnd ( poWeight );
   }

   return TRUE;
}



/* Destroy neuron. Make internal lists point to emty list and
remove weights.
**********************************************************/
void CNeuron::mDestroy ()
{
   static CNodeList
      oEmptyList;
   m_oX = oEmptyList;

   m_oW.mRemoveAll ();
}



/* Save the neuron data: net and weights on a object file
gived by oFile.
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CNeuron::mSave ( CFile& oFile )
{
   // save net
   if ( ! oFile.mSetMem ( sizeof ( float ), (void*) &m_fNet ) )
      return FALSE;

   // save weigths

   // while exist weights (selection not NULL)
   m_oW.mSelectStart ();
   while ( m_oW.mRetExistSelection () ) {

      // for each weigth
      // try save weight, if error exit function returning FALSE
      if ( ! oFile.mSetMem ( sizeof ( float ), (void*) &( (CNodeFloat*) m_oW.mRetSelected () )->m_fValue ) )
         return FALSE;

      // select next weight
      m_oW.mSelectNext ();
   }

   return TRUE;
}



/* Load the neuron data: net and weights on a object file
gived by oFile. This function does not allocate the neuron
memory, only update the values acording file data.
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CNeuron::mLoad ( CFile& oFile )
{
   // load net
   if ( ! oFile.mRetMem ( sizeof ( float ), (void*) &m_fNet ) )
      return FALSE;

   // load weigths

   // while exist weights (selection not NULL)
   m_oW.mSelectStart ();
   while ( m_oW.mRetExistSelection () ) {

      // for each weigth
      // try load weight, if error exit function returning FALSE
      if ( ! oFile.mRetMem ( sizeof ( float ), (void*) &( (CNodeFloat*) m_oW.mRetSelected () )->m_fValue ) )
         return FALSE;

      // select next weight
      m_oW.mSelectNext ();
   }

   return TRUE;
}



/* Return the tanh of the net, this value is used in propagation
of neuron that not in first layer.
**********************************************************/
float CNeuron::mRetPropagationValue ()
{
   return tanh ( m_fNet );
}



/* Return tanh(net)*(1-tanh(net)*tanh(net)), this value is used in adaptation.
**********************************************************/
float CNeuron::mRetAdaptationValue ()
{
   return tanh ( m_fNet );
}



/* 1 - Compute neuron net and update CNeuron::m_fNet with the value.
Used if neuron is not in first hiden layer,, the input is a vector
of CNodeNeuron
**********************************************************/
void CNeuron::mPropagate ()
{
   // for each neuron input
   m_oW.mSelectStart ();
   m_oX.m_oList.mSelectStart ();

   // start net with bias value
   m_fNet = ( (CNodeFloat*) m_oW.mRetSelected () )->m_fValue;
   m_oW.mSelectNext ();

   // while exist input (selection not NULL)
   while ( m_oW.mRetExistSelection () ) {

      // define input weight (W)
      CNodeFloat
         *poWeight = (CNodeFloat*) m_oW.mRetSelected ();
      // define input value (X)
      CNodeNeuron
         *poInput = (CNodeNeuron*) m_oX.m_oList.mRetSelected ();

      // compute product for this weight and input
      m_fNet += poWeight->m_fValue * poInput->m_oNeuron.mRetPropagationValue ();

      // select next weight and input
      m_oW.mSelectNext ();
      m_oX.m_oList.mSelectNext ();
   }
}



/* 2 - Compute neuron net and update CNeuron::m_fNet with the
value. Used if neuron is in first hiden layer, the input is a
vector of CNodeFloat. oInput is the vector with neuron inputs.
**********************************************************/
void CNeuron::mPropagate ( CList& oInput )
{
   // for each neuron input
   m_oW.mSelectStart ();
   oInput.mSelectStart ();

   // start net with bias value
   m_fNet = ( (CNodeFloat*) m_oW.mRetSelected () )->m_fValue;
   m_oW.mSelectNext ();

   // while exist input (selection not NULL)
   while ( m_oW.mRetExistSelection () ) {

      CNodeFloat
         // define input weight (W)
         *poWeight = (CNodeFloat*) m_oW.mRetSelected (),
         // define input value (X)
         *poInput = (CNodeFloat*) oInput.mRetSelected ();

      // compute product for this weight and input
      m_fNet += poWeight->m_fValue * poInput->m_fValue;

      // select next weight and input
      m_oW.mSelectNext ();
      oInput.mSelectNext ();
   }
}



/* Make neuron adaptation.
fRate is the adaptation rate for weights.
fDelta is the delta used to compute new weights.
oSum is used in delta definition.
**********************************************************/
void CNeuron::mAdapt ( float fRate, float fDelta, CList oSum )
{
   oSum.mSelectStart ();
   m_oX.m_oList.mSelectStart ();
   m_oW.mSelectStart ();

   // if first weight: is bias
   BOOL
      bBias = TRUE;

   // while not all weights are updated
   while ( m_oW.mRetExistSelection () ) {

      // define delta
      float
         fInput,
         fDeltaW;

      // define input
      // verify if is bias input, if yes always is 1
      if ( bBias )
         fInput = 1;
      else
         fInput = ( (CNodeNeuron*) m_oX.m_oList.mRetSelected () )->m_oNeuron.mRetAdaptationValue ();

      // delta W
      fDeltaW = fRate * fDelta * fInput;

      // define W
      ( (CNodeFloat*) m_oW.mRetSelected () )->m_fValue += fDeltaW;

      // define sum if not bias
      if ( ! bBias )
         ( (CNodeFloat*) oSum.mRetSelected () )->m_fValue += fDelta *
                    ( (CNodeFloat*) m_oW.mRetSelected () )->m_fValue;

      // next
      m_oW.mSelectNext ();
      // next only if is not bias, bias is not realy a link
      if ( ! bBias ) {
         m_oX.m_oList.mSelectNext ();
         oSum.mSelectNext ();
      }
      else
         bBias = FALSE;
   }
}



/* Make neuron adaptation. Called for adaptation of
neurons in last layer (from back to front).
Do not compute sum.
fRate is the adaptation rate for weights.
fDelta is the delta used to compute new weights.
oInput is the train vector.
**********************************************************/
void CNeuron::mAdaptLast ( float fRate, float fDelta, CList oInput )
{
   oInput.mSelectStart ();
   m_oW.mSelectStart ();

   // if first weight: is bias
   BOOL
      bBias = TRUE;

   // while not all weights are updated
   while ( m_oW.mRetExistSelection () ) {

      // define delta W
      float
         fInput,
         fDeltaW;

      // define input
      // verify if is bias input allways 1
      if ( bBias )
         fInput = 1;
      else
         fInput = ( (CNodeFloat*) oInput.mRetSelected () )->m_fValue;

      // delta W
      fDeltaW = fRate * fDelta * fInput;

      // define W
      ( (CNodeFloat*) m_oW.mRetSelected () )->m_fValue += fDeltaW;

      // next
      m_oW.mSelectNext ();
      // next only if is not bias, bias is not realy a link
      if ( ! bBias )
         oInput.mSelectNext ();
      else
         bBias = FALSE;
   }
}
