/* Store polygon data: what vertexes compose it, surface data,
plan equation, ...
*/



#include <string.h>

#include "defs.h"

#include "node.h"

#include "nodeflo.h"

#include "list.h"

#include "polygon.h"



/* constructor - init empty poly
**********************************************************/
CPolygon::CPolygon ()
{
   m_padwVertexes = NULL;
   m_dwNumVertexes = 0;
   m_dwMaxVertexes = 0;
   m_stPortal.yCode = 0;
   m_stPortal.dwSectorID = 0;
   m_stPortal.poSector = NULL;
   m_dwLightID = 0;
   m_yCrossable = 0;
   m_dwTextureID = (DWORD) -1;
   m_fLight = DEFAULT_LIGHT_LEVEL;
   m_stNormal.fX = m_stNormal.fY = m_stNormal.fZ = 0;
   m_fD = 0;
}



/* destructor - desalocate Polygon data, if alocated
**********************************************************/
CPolygon::~CPolygon ()
{
   mDestroy ();
}



/* Create a new poly using the vertexes given from oIndexes (CNodeInt)
dwLightID - the light table to use when drawing poly surface
fLight - the poly light level (between 0 and 1)
**********************************************************/
void CPolygon::mCreate ( CList& oIndexes, DWORD dwLightID, float fLight )
{
   // default value for attributes
   m_stPortal.yCode = 0;
   m_stPortal.dwSectorID = 0;
   m_stPortal.poSector = NULL;
   m_yCrossable = 0;
   m_dwTextureID = (DWORD) -1;
   m_dwLightID = dwLightID;
   m_fLight = fLight;
   m_stNormal.fX = m_stNormal.fY = m_stNormal.fZ = 0;
   m_fD = 0;

   // verify if need realocate memory
   if ( m_dwMaxVertexes < oIndexes.mRetNumNodes () ) {
      delete m_padwVertexes;
      // try create the vertex index array
      m_padwVertexes = new DWORD [ oIndexes.mRetNumNodes () ];
      if ( m_padwVertexes == NULL )
         ABORT ( MSG_ERROR_MEMORY_LOW, __FILE__, __LINE__ )
      m_dwMaxVertexes = oIndexes.mRetNumNodes ();
   }

   // copy vertexes
   m_dwNumVertexes = oIndexes.mRetNumNodes ();
   // scan all list indexes and copy them to vertexes index array
   for ( DWORD dwcI = 0; dwcI < m_dwNumVertexes; dwcI ++ )
      m_padwVertexes [ dwcI ] = (DWORD) ( (CNodeFloat*) oIndexes [ dwcI ] )->m_fValue;
}



/* Desalocate Polygon data, if alocated
**********************************************************/
void CPolygon::mDestroy ()
{
   if ( m_padwVertexes != NULL ) {
      delete m_padwVertexes;
      m_padwVertexes = NULL;
      m_dwNumVertexes = 0;
      m_dwMaxVertexes = 0;
   }
}



/* Copy one polygon to this
**********************************************************/
void CPolygon::mSetObj ( CPolygon& oPol )
{
   // verify if need realocate memory
   if ( m_dwMaxVertexes < oPol.m_dwNumVertexes ) {
      delete m_padwVertexes;
      // try create the vertex index array
      m_padwVertexes = new DWORD [ oPol.m_dwNumVertexes ];
      if ( m_padwVertexes == NULL )
         ABORT ( MSG_ERROR_MEMORY_LOW, __FILE__, __LINE__ )
      m_dwMaxVertexes = oPol.m_dwNumVertexes;
   }

   // copy vertexes
   memcpy ( (void*) m_padwVertexes, (void*) oPol.m_padwVertexes, sizeof ( DWORD ) * oPol.m_dwNumVertexes );
   m_dwNumVertexes = oPol.m_dwNumVertexes;

   m_stPortal.yCode = oPol.m_stPortal.yCode;
   m_stPortal.dwSectorID = oPol.m_stPortal.dwSectorID;
   m_stPortal.poSector = oPol.m_stPortal.poSector;
   m_yCrossable = oPol.m_yCrossable;
   m_dwTextureID = oPol.m_dwTextureID;
   m_fLight = oPol.m_fLight;
   m_dwLightID = oPol.m_dwLightID;
   m_stNormal.fX = oPol.m_stNormal.fX;
   m_stNormal.fY = oPol.m_stNormal.fY;
   m_stNormal.fZ = oPol.m_stNormal.fZ;
   m_fD = oPol.m_fD;
}



/* like CPolygon::mSetObj
**********************************************************/
void CPolygon::operator = ( CPolygon& oPol )
{
   mSetObj ( oPol );
}   
