/* Declare scenery.cc */



// verify if has been declared, if no do it
#ifndef __3DFC_SCENERY_H__



// signals file inclusion
#define __3DFC_SCENERY_H__



class CScenery {

public:

   CScenery ();
   CScenery ( CString oszFile );
   ~CScenery ();

private:

   // used only when creating the scenery
   DWORD m_dwDefineID;       // ID of current define data
   DWORD m_dwSectorID;       // ID of current sector beeing defined
   BOOL m_bDefining;         // if TRUE a define block beeing defined else a
                             // sector.
                             // With this polygons, vertexes and attributes
                             // beeing added refer to define block (if TRUE)
                             // or to sector (if FALSE).

   // default parameters when adding data
   float m_fDefaultScaleFactor;       // for vertexes
   float m_fDefaultLightLevel;        // between 0 and 1
   DWORD m_dwDefaultLightTable;       

public:

   BOOL mCreate ( CString oszFile );
   void mDestroy ();

   // scenery data
   CMem m_oPal;               // palette
   CMap m_oLightTables;       // light tables (CNodeLightTable)
   CMap m_oTextures;          // texture set (CNodeTexture)
   CMap m_oDefines;           // definitions available for instanciation (CNodeObject)
   CNodeSector m_oRoot;       // object instances in first (top) sector
   
   // message data 
   CEventTable m_oEventTab;  // messages to send in each interaction

   // frame buffer configuration
   INT  m_nClearBackground;  // background color to clear frame buff (-1 no clear)

   // camera configuration
   CNodeObject* m_poCamera;  // object instance that is the camera

   // end program
   BOOL m_bEnd;              // flag if need end the program (TRUE - end)

   // resources
   BOOL mAddPalette ( CMem oPal );
   BOOL mAddLightTable ( CLightTable& oLgT, DWORD dwID, CString oszFile,
                         DWORD dwNumLevels, float fR, float fG, float fB );
   BOOL mAddTexture ( DWORD dwID, CTexture& oTx );

   // instances
   BOOL mAddObject ( DWORD dwInstID, DWORD dwDestinInstID,
                     DWORD dwDestinSectorID, CList& oDefs );

   // messages
   INT mAddMessage ( CNodeObject* poObj, DWORD dwObjID, DWORD dwSlotID, PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 );
   void mRemoveMessage ( DWORD dwPos );

   // start configuration
   BOOL mSetStart ( DWORD dwCameraID, INT nClear );
   
   // defines
   BOOL mBeginAddDefine ( DWORD dwDefID );
   BOOL mEndAddDefine ();
   BOOL mBeginAddSector ( DWORD dwSectorID );
   BOOL mEndAddSector ();
   BOOL mAddPolygon ( CList& oIndexes );
   BOOL mAddVertex ( float fX, float fY, float fZ );
   BOOL mSetCurrentCamera ( DWORD dwCameraID );
   BOOL mSetDataValues ( INT nTypeID, float* pafData, DWORD dwNdxPol1, DWORD dwNdxPol2 );
   void mSetLightTable ( DWORD dwLightID );
   void mSetLightLevel ( float fLight );
   
   // components
   BOOL mCreateComponent ( DWORD dwSlotID, DWORD dwTypeID, PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 );
   BOOL mActivateComponent ( DWORD dwSlotID, PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 );
   BOOL mDestroyComponent ( DWORD dwSlotID, PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 );
};



// end declarations
#endif
