/* Store a sector data: polygon mesh and instances of objects inside this sector.
A sector must define a convex group of polys in scenery.

TIPS:
- this class derive from CMesh, see CMesh for more info
                
WARNING:
- after change sector data call CSector::mBeginAddSector and imediatly before
  call CSector::mEndAddSector.
*/



#include <string.h>

#include "defs.h"

#include "node.h"

#include "list.h"

#include "polygon.h"

#include "nodepol.h"

#include "nodeint.h"

#include "map.h"

#include "math.h"

#include "mesh.h"

#include "refsys.h"

#include "camsys.h"

#include "object.h"

#include "nodeobj.h"

#include "sector.h"

#include "comp/nodecomp.h"

#include "comp/cosecset.h"




/* constructor - init empty sector
**********************************************************/
CSector::CSector ()
{
   m_poObject = NULL;
}



/* destructor - desalocate sector data, if alocated
**********************************************************/
CSector::~CSector ()
{
   mDestroy ();
}



/* Desalocate sector data, if alocated
**********************************************************/
void CSector::mDestroy ()
{
   CMesh::mDestroy ();

   m_poObject = NULL;
   m_oObjects.mRemoveAll ();
}



/* Prepare sector to reseve data.
Call after change data, before call CSector::mEndAddSector
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CSector::mBeginAddSector ()
{
   return mBeginAddMesh ();
}



/* Update sector data acording definition.
Call after define all data and before use this data.
return TRUE - ok / FALSE - error
**********************************************************/
BOOL CSector::mEndAddSector ()
{
   return mEndAddMesh ();
}



/* Search in this sector for a given object instance and, if
   not found, in the objects that are inside this sector.
return the object pointer or NULL if not found.
**********************************************************/
CNodeObject* CSector::mSearchObject ( DWORD dwObjID )
{
   CNodeObject
      *poObj,
      *poObjFound;

   // scan objects vailable
   DWORD
      dwNum = m_oObjects.mRetSize ();
   for ( DWORD dwcInd = 0; dwcInd < dwNum; dwcInd ++ ) {

      // for each key (object) in sector
      DWORD
         dwKey = m_oObjects.mRetKey ( dwcInd );
      poObj = (CNodeObject*) m_oObjects.mRetValue ( dwKey );

      // verify if is the same key, if yes object found
      if ( dwKey == dwObjID )

         // return the object
         return poObj;         

      else { // search ID inside the sectors of this object

         // get the sectors component of the object
         CComponentSectorsSet
            *poSectors = (CComponentSectorsSet*) poObj->m_oObject.mRetComponent ( SLOT_INTERIOR );
         // verify if component not found -> skip this object
         if ( poSectors == NULL )
            continue;

         // search
         if ( ( poObjFound = (CNodeObject*) poSectors->mActivate ( 4, dwObjID, 0 ) ) != NULL )
            return poObjFound;
      }
   }

   // if not found
   return NULL;
}
                  
