/* Implement object string ended by 0. The string is an array of bytes ended
by 0.

TIPS:
- index positions start at 0
- to know if string is empty use the comparison CString::mRetLength () == 0
- this class is derived from CMem. See CMem for more info.
- to make a scanf with a CString object first pre-alocate some bytes in
  buffer using CString::mCreate, the constructor, ... Example:
    CString
      oStr ( 100 );
    scanf ( "%s", (char*) oStr );
*/



#include <string.h>

#include "defs.h"

#include "mem.h"

#include "string.h"



/* constructor - string empty (only end 0)
**********************************************************/
CString::CString ()
{
   mCreate ( 1 );

   m_payMem [ 0 ] = 0;
}



/* constructor - init string with copy of other object
**********************************************************/
CString::CString ( CString& oStr )
{
   mSetObj ( oStr );
}



/* constructor - init empty string with a prealocated lenght
reserved. Automaticaly reserve space for end 0.
**********************************************************/
CString::CString ( DWORD dwSize )
{
   mCreate ( dwSize + 1 );   // 1 for 0 end

   m_payMem [ 0 ] = 0;
}



/* constructor - init string with array of char + 0 end
**********************************************************/
CString::CString ( const char* szStr )
{
   // set object
   *this = szStr;
}



/* destructor - desalocate memory if alocated
**********************************************************/
CString::~CString ()
{
   mDestroy ();
}



/* Alocate space for string. No reserve space for 0 end.
**********************************************************/
void CString::mCreate ( DWORD dwSize )
{
   mDestroy ();

   CMem::mCreate ( dwSize );
}



/* Empty string
**********************************************************/
void CString::mDestroy ()
{
   // set first char to 0 if exist
   if ( m_payMem != NULL )
      m_payMem [ 0 ] = 0;
   m_dwSizeUsed = 0;
}



/* Init string with copy of other object, they will have the same
memory size. (like CString::construtor and CString::operator =)
**********************************************************/
void CString::mSetObj ( CString& oStr )
{
   // verify if is necessary alocate more memory
   if ( m_dwSize < oStr.m_dwSize )
      mCreate ( oStr.m_dwSize );

   // copy memory (include 0 end)
   memcpy ( (void*) m_payMem, (void*) oStr.m_payMem, m_dwSize );

   m_dwSizeUsed = oStr.mRetLength ();
}



/* Return length of string, without 0 end
**********************************************************/
DWORD CString::mRetLength ()
{
   return m_dwSizeUsed;
}



/* 1 - like CString::constructor and CString::mSetObj
**********************************************************/
void CString::operator = ( CString& oStr )
{
   mSetObj ( oStr );
}



/* 2 - Init string with copy of other char array (like CString::constructor)
**********************************************************/
void CString::operator = ( const char* szStr )
{
   // verify if is necessary alocate more memory
   DWORD
      dwSize = strlen ( szStr );
   if ( m_dwSize < dwSize + 1 )
      mCreate ( dwSize + 1 );    // include 0 end

   // copy string with 0 end
   strcpy ( (char*) m_payMem, szStr );

   // define new dimensions
   m_dwSizeUsed = dwSize;
}



/* Return a char's string, start dwPos in 0
**********************************************************/
char CString::operator [] ( DWORD dwPos )
{
   return * ( m_payMem + dwPos );
}



/* Cast object to array of char ending with 0
**********************************************************/
CString::operator char* ()
{
   return (char*) m_payMem;
}



/* 1 - Concatenate with other object objeto and return
concatenation as array of char with 0 end
**********************************************************/
char* CString::operator + ( CString& oStr )
{
   // Create object with concatenation of this object and the give object
   static CString
      oStrRes ( oStr.mRetLength () + m_dwSizeUsed + 1 ); // reserve 0 end

   // insert first string
   oStrRes = *this;

   // concatenate with second string
   oStrRes += oStr;

   return (char*) oStrRes.m_payMem;
}



/* 2 - Concatenate with array of char ended by 0 and return
concatenation as array of char with 0 end
**********************************************************/
char* CString::operator + ( const char* szStr )
{
   // Create object with concatenation of this object and the give object
   static CString
      oStrRes ( strlen ( szStr ) + m_dwSizeUsed + 2 ); // need 2 0 end!

   // insert first string
   oStrRes = *this;

   // concatenate with second string
   oStrRes += szStr;

   return (char*) oStrRes.m_payMem;
}



/* 1 - Concatenate with other object
**********************************************************/
void CString::operator += ( CString& oStr )
{
   DWORD
      dwPosEnd = m_dwSizeUsed;     // last used position

   // verify if actual size memory is big enough
   if ( m_dwSize < oStr.mRetLength () + m_dwSizeUsed )
      mRealocate ( m_dwSize + oStr.mRetLength () + 1 );     // alocate more

   // copy new string
   memcpy ( (void*) ( m_payMem + dwPosEnd ), (void*) oStr.m_payMem, oStr.mRetLength () );

   // define new lenght
   m_dwSizeUsed += oStr.mRetLength ();

   // set 0 end
   m_payMem [ m_dwSizeUsed ] = 0;
}



/* 2 - Concatenate with array of char ended by 0
**********************************************************/
void CString::operator += ( const char* szStr )
{
   DWORD
      dwPosEnd = m_dwSizeUsed;     // last used position

   // verify if actual size memory is big enough
   DWORD
      dwSize = strlen ( szStr );
   if ( m_dwSize < dwSize + m_dwSizeUsed )
      mRealocate ( m_dwSize + dwSize + 1 );     // alocate more

   // copy new string
   memcpy ( (void*) ( m_payMem + dwPosEnd ), (void*) szStr, dwSize );

   // define new lenght
   m_dwSizeUsed += dwSize;

   // set 0 end
   m_payMem [ m_dwSizeUsed ] = 0;
}



/* Split object in 2 string. Search from end to start the given
chC char, when found split string in it's position. The left side
stay in this object and the right go to oStr. Stop search in first
ocurrence found. return TRUE - char found / FALSE - not found
**********************************************************/
BOOL CString::mSplitFromRight ( char chC, CString& oStr )
{
   // search for char
   for ( INT ncP = (INT) m_dwSizeUsed - 1; ncP >= 0; ncP -- )

      // if found one char
      if ( m_payMem [ ncP ] == chC ) {

         // create new left string
         CString
            oStrLeft;
         // if left have some char
         if ( ncP > 0 ) {
            oStrLeft.mCreate ( ncP + 1 );  // 0 end
            memcpy ( (void*) oStrLeft.m_payMem, (void*) m_payMem, ncP );
            oStrLeft.m_dwSizeUsed = (DWORD) ncP;
            oStrLeft.m_payMem [ oStrLeft.m_dwSizeUsed ] = 0;
         }

         // create new right string
         // if right have some char
         if ( ncP < (INT) mRetLength () ) {
            DWORD
               dwRightSize = mRetLength () - ( (DWORD) ncP + 1 );
            oStr.mCreate ( dwRightSize + 1 );  // 0 end
            memcpy ( (void*) oStr.m_payMem, (void*) ( m_payMem + ncP + 1 ), dwRightSize );
            oStr.m_dwSizeUsed = dwRightSize;
            oStr.m_payMem [ oStr.m_dwSizeUsed ] = 0;
         }

         // assign left to this object
         *this = oStrLeft;

         return TRUE;
      }

   oStr.mDestroy ();

   return FALSE;
}



/* Split object in 2 string. Search from start to end the given
chC char, when found split string in it's position. The left side
stay in this object and the right go to oStr. Stop search in first
ocurrence found. return TRUE - char found / FALSE - not found
**********************************************************/
BOOL CString::mSplitFromLeft ( char chC, CString& oStr )
{
   // search for char
   for ( DWORD dwcP = 0; dwcP <= m_dwSizeUsed; dwcP ++ )

      // if found one char
      if ( m_payMem [ dwcP ] == chC ) {

         // create new left string
         CString
            oStrLeft;
         // if left have some char
         if ( dwcP > 0 ) {
            oStrLeft.mCreate ( dwcP + 1 );  // 0 end
            memcpy ( (void*) oStrLeft.m_payMem, (void*) m_payMem, dwcP );
            oStrLeft.m_dwSizeUsed = dwcP;
            oStrLeft.m_payMem [ oStrLeft.m_dwSizeUsed ] = 0;
         }

         // create new right string
         // if right have some char
         if ( dwcP < mRetLength () ) {
            DWORD
               dwRightSize = mRetLength () - ( dwcP + 1 );
            oStr.mCreate ( dwRightSize + 1 );  // 0 end
            memcpy ( (void*) oStr.m_payMem, (void*) ( m_payMem + dwcP + 1 ), dwRightSize );
            oStr.m_dwSizeUsed = dwRightSize;
            oStr.m_payMem [ oStr.m_dwSizeUsed ] = 0;
         }

         // assign left to this object
         *this = oStrLeft;

         return TRUE;
      }

   oStr.mDestroy ();

   return FALSE;
}



/* Compare 2 CString objects, if they are equal return TRUE
else FALSE, no case sensitive
**********************************************************/
BOOL CString::operator == ( CString oStr )
{
   // test without case sensitive
   if ( strcasecmp ( (char*) oStr, (char*) m_payMem ) == 0 )
      return TRUE;

   return FALSE;
}



/* Compare 2 CString objects, if they are equal return TRUE
else FALSE, no case sensitive
**********************************************************/
BOOL CString::operator != ( CString oStr )
{
   // test without case sensitive
   if ( strcasecmp ( (char*) oStr, (char*) m_payMem ) == 0 )
      return FALSE;

   return TRUE;
}



/* Seach if character chC exist in string, start left.
return TRUE - exist / FALSE no
**********************************************************/
BOOL CString::mTestFromLeft ( char chC )
{
   // search for char
   for ( DWORD dwcP = 0; dwcP <= m_dwSizeUsed; dwcP ++ )

      // if found one char return find
      if ( m_payMem [ dwcP ] == chC )
         return TRUE;

   return FALSE;
}


/* Replace all characters chOrig with chNew
**********************************************************/
void CString::mReplace ( char chOrig, char chNew )
{
   // search for char scanning all string
   for ( DWORD dwcP = 0; dwcP <= m_dwSizeUsed; dwcP ++ )

      // if found one char, replace
      if ( m_payMem [ dwcP ] == chOrig )
         m_payMem [ dwcP ] = chNew;         
}   
