/* Implementa funcoes de handler de teclado para deteccao de varias teclas
pressionadas simultaneamente. Eh usado para substituir a funcao de
tratamento do teclado da BIOS (handler de teclado da BIOS) por uma outra
funcao fornecida pelo programa.

As funcoes possuem 2 implementacoes: dentro da classe e fora (para depuracao)
As funcoes de fora da classe soh funcionam se jah houver um objeto da classe
instanciado. Nao pode haver mais de 1 instancia da classe ao mesmo tempo
(pode causar problemas no handler).

TODAS AS TECLAS DEVEM ESTAR SOLTAS QUANDO O HANDLER FOR REMOVIDO, SENAO GPF
NO WINDOWS!!!

Numero da interrupcao do teclado: 9
Porta para leitura de teclas: 0x60
Porta para aviso de recebimento de interrupcao: 0x20, valor 0x20
Codigo dos bytes enviados para o handler (nao tratado PrtScr e Ctrl+Break):
 Se for tecla especial:
   valor 224
   Byte seguinte (na proxima chamada ao handler):
   Se for tecla sendo pressionada
     valor codigo_tecla        (posicao na tabela: codigo_tecla + 128)
   Se for tecla sendo solta
     valor codigo_tecla + 128  (posicao na tabela: codigo_tecla + 128)
 Se nao for tecla especial
   Se for tecla sendo pressionada
     valor codigo_tecla
   Se for tecla sendo solta
     valor codigo_tecla + 128                                              */



#include <dpmi.h>
#include <go32.h>
#include <string.h>
#include <dos.h>

#include "defs.h"

// Devem ser globais para handler acessar. Se nao forem globais alteracoes
// feitas nao sao visiveis para demais funcoes locais.
BOOL ayTECLAS [ 256 ];    // indica tecla pressionada, TRUE, ou nao, FALSE
BOOL bESPECIAL = FALSE;   // indica tecla especial sendo tratada

#include "teclado.h"

#include <stdio.h>



/* construtor - Se bInstalarHandler = TRUE substitui a
funcao de tratamento do teclado atual pela funcao do programa.
Default bInstalarHandler = FALSE
**********************************************************/
CTeclado::CTeclado ( BOOL bInstalarHandler )
{
   m_bInstalarHandler = bInstalarHandler;

	// verifica se deve instalar handler
   if ( m_bInstalarHandler ) {

      // antes de substituir pelo novo handler eh necessario salvar o antigo
	   _go32_dpmi_get_protected_mode_interrupt_vector ( 9, &m_stHandlerAntigo );

      // reserva memoria para as funcoes do handler e chamadas pelo handler
	   //  (impossibilita swap como memoria virtual)
      _go32_dpmi_lock_code ( (void*) mHandlerNovo, (UINT) mHandlerNovo___ - (UINT) mHandlerNovo );

      // reserva memoria do handler (impossibilita swap como memoria virtual)
      _go32_dpmi_lock_data ( ayTECLAS, sizeof ( ayTECLAS ) );

      // substitui o handler de teclado atual pelo fornecido pelo programa
	   m_stHandlerNovo.pm_offset = (INT) mHandlerNovo;
	   m_stHandlerNovo.pm_selector = _go32_my_cs();
	   _go32_dpmi_allocate_iret_wrapper ( &m_stHandlerNovo );
	   _go32_dpmi_set_protected_mode_interrupt_vector ( 9, &m_stHandlerNovo );

      // limpa a tabela de teclas pressionadas
      memset ( (void*) ayTECLAS, FALSE, sizeof ( ayTECLAS ) );
   }
}



/* destrutor - substitui o handler do programa pelo original.
Agaurda ateh todas as teclas estarem soltas para isso.
**********************************************************/
CTeclado::~CTeclado ()
{
	// verifica se deve desinstalar handler
   if ( m_bInstalarHandler ) {

      mRetSoltas ();

      // substitui o handler de teclado atual pelo antigo
	   _go32_dpmi_set_protected_mode_interrupt_vector ( 9, &m_stHandlerAntigo );

      // libera memoria do handler do programa
	   _go32_dpmi_free_iret_wrapper ( &m_stHandlerNovo );
   }
}



/* Handler de telcado (captura teclas), chamado cada vez que
uma tecla eh pressionada ou solta, ou seja, sempre que eh
gerada a interrupcao do teclado (numero 9).
VER mFimHandler___!!!!!!!!!
**********************************************************/
void CTeclado::mHandlerNovo ()
{
   unsigned char
		yTecla = inportb ( 0x60 );

   // exibe informacoes sobre a tecla
   // se eh especial
   if ( yTecla == 224 )
   	printf ( "especial " );
   else
	   // verifica se nao eh a chama apohs 1o. byte de tecla especial
   	if ( ! ( bESPECIAL && yTecla != 224 ) )
	   	printf ( "         " );
   // se estah sendo solta
   // verifica se nao eh a chama do 1o. byte de tecla especial
  	if ( yTecla != 224 )
	   if ( ( yTecla & 128 ) == 0 )
   		printf ( "press " );
	   else
   		printf ( "solta " );
   // codigo da tecla
   if ( yTecla != 224 )
   	printf ( "%u\n", ( yTecla & 127 ) );

   // verifica se nao foi pressionado anteriormente um tecla especial
   if ( ! bESPECIAL )
   	// detecta pressionamento de tecla especial
      if ( yTecla == 224 )
      	bESPECIAL = TRUE;
      else
   	   // trata tecla nao especial
         // Altera posicao na tabela correspondente a tecla pressionada
         // se tecla estah sendo pressionada (bit 7 == 0) altera posicao
	      // correspondente para TRUE senao para FALSE.
         if ( ( yTecla & 128 ) == 0 )
	         * ( ayTECLAS + ( yTecla & 127 ) ) = TRUE;
         else
	         // senao altera para FALSE
	         * ( ayTECLAS + ( yTecla & 127 ) ) = FALSE;
   else {
   	// trata tecla especial
      // se foi pressionada
      if ( ( yTecla & 128 ) == 0 )
	      * ( ayTECLAS + 128 + ( yTecla & 127 ) ) = TRUE;
      else
	      // se foi solta
	      * ( ayTECLAS + 128 + ( yTecla & 127 ) ) = FALSE;

      bESPECIAL = FALSE;
   }

   // confirma recebimento da interrupcao
	outportb ( 0x20, 0x20 );
}



/* Usado apenas como referencia para saber tamanho do
codigo executavel da funcao mHandlerNovo: (mHandlerNovo___ -
- mHandlerNovo). DEVE VIR SEMPRE LOGO APOS mHandlerNovo!!!
**********************************************************/
void CTeclado::mHandlerNovo___ ()
{
}



/* Verifica se uma determinada tecla estah pressionada
(return TRUE) ou nao (return FALSE)
**********************************************************/
BOOL CTeclado::mRetPress ( BYTE yTecla )
{
	return * ( ayTECLAS + yTecla );
}



/* Aguarda ateh que alguma tecla seja pressionada. return
codigo da tecla pressionada. Avalia cada tecla do vetor de
teclas em sequencia ateh o seu fim e entao volta ao inicio
e repete. Se bPress = TRUE somente comeca a testar teclas
apos todas estarem soltas, se for FALSE e uma tecla jah estava
pressionada entao fica no loop apenas 1 volta e pode confundir
usuario por ser muito rapido.
**********************************************************/
BYTE CTeclado::mRetTeclas ( BOOL bTestaPress )
{
	// Se todas as teclas devem estar soltas antes de aguardar que alguma
   // tecla seja pressionada.
   if ( bTestaPress ) {
     	BOOL
        	bSoltas;
	   // enquanto uma tecla estiver pressionada
      do {
        	bSoltas = TRUE;
	      // varre todo vetor de teclas
			for ( UINT16 ucT = 0; ucT < 256; ucT ++ )
      	   // se encontrou alguma tecla pressionada (TRUE)
   			if ( mRetPress ( ucT ) == TRUE )
         		bSoltas = FALSE;
   	} while ( bSoltas == FALSE );
   }

   // enquanto uma tecla nao for pressionada
   while ( 1 )
      // varre todo vetor de teclas
		for ( UINT16 ucT = 0; ucT < 256; ucT ++ )
         // se encontrou alguma tecla pressionada (TRUE) retorn seu codigo
   		if ( mRetPress ( ucT ) == TRUE )
         	return ucT;
}



/* Aguarda ateh que a tecla yTecla seja pressionada.
Se bPress = TRUE somente comeca a testar a tecla
apos essa tecla estar solta, se for FALSE e a tecla jah estava
pressionada entao fica no loop apenas 1 volta e pode confundir
usuario por ser muito rapido.
**********************************************************/
void CTeclado::mRetTecla ( BYTE yTecla, BOOL bTestaPress )
{
   // se eh para testar se tecla estava pressionada antes
   if ( bTestaPress )
      // enquanto a tecla estiver pressionada
      while ( mRetPress ( yTecla ) == TRUE );

   // enquanto a tecla nao for pressionada
   while ( mRetPress ( yTecla ) == FALSE );;
}



/* Aguarda ateh todas as teclas estarem soltas
**********************************************************/
void CTeclado::mRetSoltas ()
{
   BOOL
      bSoltas;
	// enquanto uma tecla estiver pressionada
   do {
      bSoltas = TRUE;
	   // varre todo vetor de teclas
		for ( UINT16 ucT = 0; ucT < 256; ucT ++ )
      	// se encontrou alguma tecla pressionada (TRUE)
   		if ( mRetPress ( ucT ) == TRUE )
         	bSoltas = FALSE;
   } while ( bSoltas == FALSE );
}
